@@ -52,27 +52,47 @@ static unsigned int assess_danger(struct city *pcity);
   Choose the best unit the city can build to defend against attacker v.
 **************************************************************************/
 struct unit_type *ai_choose_defender_versus(struct city *pcity,
-					    const struct unit_type *v)
+					    struct unit *attacker)
 {
   struct unit_type *bestunit = NULL;
-  int best = 0;
+  double best = 0;
+  int best_cost = FC_INFINITY;
+  struct player *pplayer = city_owner(pcity);
 
   simple_ai_unit_type_iterate(punittype) {
-    const int move_type = punittype->move_type;
+    const int move_type = utype_move_type(punittype);
 
-    if (can_build_unit(pcity, punittype)
+    if (can_city_build_unit_now(pcity, punittype)
 	&& (move_type == LAND_MOVING || move_type == SEA_MOVING)) {
-      const int defense = get_virtual_defense_power(v, punittype,
-						    city_owner(pcity),
-						    pcity->tile,
-						    FALSE, FALSE);
+      int fpatt, fpdef, defense, attack;
+      double want, loss, cost = utype_build_shield_cost(punittype);
+      struct unit *defender;
+      int veteran = get_unittype_bonus(city_owner(pcity), pcity->tile, punittype,
+                                       EFT_VETERAN_BUILD);
 
-      if (defense > best || (defense == best
-			     && unit_build_shield_cost(punittype) <=
-			     unit_build_shield_cost(bestunit))) {
-        best = defense;
+      defender = create_unit_virtual(pplayer, pcity, punittype, veteran);
+      defense = get_total_defense_power(attacker, defender);
+      attack = get_total_attack_power(attacker, defender);
+      get_modified_firepower(attacker, defender, &fpatt, &fpdef);
+
+      /* Greg's algorithm. loss is the average number of health lost by
+       * defender. If loss > attacker's hp then we should win the fight,
+       * which is always a good thing, since we avoid shield loss. */
+      loss = (double) defense * punittype->hp * fpdef / (attack * fpatt);
+      want = (loss + MAX(0, loss - attacker->hp)) / cost;
+
+#ifdef NEVER
+      CITY_LOG(LOG_DEBUG, pcity, "desire for %s against %s(%d,%d) is %.2f",
+               unit_name_orig(punittype), unit_name_orig(unit_type(attacker)), 
+               TILE_XY(attacker->tile), want);
+#endif
+
+      if (want > best || (want == best && cost <= best_cost)) {
+        best = want;
         bestunit = punittype;
+        best_cost = cost;
       }
+      destroy_unit_virtual(defender);
     }
   } simple_ai_unit_type_iterate_end;
 
@@ -107,12 +127,12 @@ static struct unit_type *ai_choose_attacker(struct city *pcity,
 
   simple_ai_unit_type_iterate(punittype) {
     cur = ai_unit_attack_desirability(punittype);
-    if (which == punittype->move_type) {
-      if (can_build_unit(pcity, punittype)
+    if (which == utype_move_type(punittype)) {
+      if (can_city_build_unit_now(pcity, punittype)
           && (cur > best
               || (cur == best
-                  && unit_build_shield_cost(punittype)
-                     <= unit_build_shield_cost(bestid)))) {
+                  && utype_build_shield_cost(punittype)
+                     <= utype_build_shield_cost(bestid)))) {
         best = cur;
         bestid = punittype;
       }
@@ -146,17 +166,17 @@ static struct unit_type *ai_choose_bodyguard(struct city *pcity,
     }
 
     /* Only consider units of same move type */
-    if (punittype->move_type != move_type) {
+    if (utype_move_type(punittype) != move_type) {
       continue;
     }
 
     /* Now find best */
-    if (can_build_unit(pcity, punittype)) {
+    if (can_city_build_unit_now(pcity, punittype)) {
       const int desire = ai_unit_defence_desirability(punittype);
 
       if (desire > best
-	  || (desire == best && unit_build_shield_cost(punittype) <=
-	      unit_build_shield_cost(bestid))) {
+	  || (desire == best && utype_build_shield_cost(punittype) <=
+	      utype_build_shield_cost(bestid))) {
         best = desire;
         bestid = punittype;
       }
@@ -220,8 +240,9 @@ int assess_defense_quadratic(struct city *pcity)
    * wallvalue = 40, walls = 20,
    * wallvalue = 90, walls = 30 */
 
-  while (walls * walls < pcity->ai.wallvalue * 10)
+  while (walls * walls < pcity->ai->wallvalue * 10) {
     walls++;
+  }
 
   unit_list_iterate(pcity->tile->units, punit) {
     defense += base_assess_defense_unit(pcity, punit, igwall, FALSE,
@@ -245,7 +266,7 @@ One unit only, mostly for findjob; handling boats correctly. 980803 -- Syela
 int assess_defense_unit(struct city *pcity, struct unit *punit, bool igwall)
 {
   return base_assess_defense_unit(pcity, punit, igwall, TRUE,
-				  pcity->ai.wallvalue);
+				  pcity->ai->wallvalue);
 }
 
 /********************************************************************** 
@@ -411,7 +432,7 @@ void assess_danger_player(struct player *pplayer)
   that can whack us, so let's build something that can defend against
   him. If danger is urgent and overwhelming, danger is 200+, if it is
   only overwhelming, set it depending on danger. If it is underwhelming,
-  set it to 100 pluss urgency.
+  set it to 100 plus urgency.
 
   This algorithm is very strange. But I created it by nesting up
   Syela's convoluted if ... else logic, and it seems to work. -- Per
@@ -457,30 +478,34 @@ static void ai_reevaluate_building(struct city *pcity, int *value,
 static unsigned int assess_danger(struct city *pcity)
 {
   int i;
-  int danger[5], defender;
+  int defender;
   struct player *pplayer = city_owner(pcity);
   bool pikemen = FALSE;
   unsigned int urgency = 0;
   int igwall_threat = 0;
   struct tile *ptile = pcity->tile;
+  int defense;
+  unsigned int danger_reduced[B_LAST]; /* How much such danger there is that
+                                          building would help against */
+  int total_danger = 0;
 
   TIMING_LOG(AIT_DANGER, TIMER_START);
 
-  memset(&danger, 0, sizeof(danger));
+  memset(&danger_reduced, 0, sizeof(danger_reduced));
 
   generate_warmap(pcity, NULL);	/* generates both land and sea maps */
 
   if (ai_handicap(pplayer, H_DANGER)) {
     /* Always thinks that city is in grave danger */
-    pcity->ai.grave_danger = 1;
+    pcity->ai->grave_danger = 1;
   } else {
-    pcity->ai.grave_danger = 0;
+    pcity->ai->grave_danger = 0;
   }
-  pcity->ai.diplomat_threat = FALSE;
-  pcity->ai.has_diplomat = FALSE;
+  pcity->ai->diplomat_threat = FALSE;
+  pcity->ai->has_diplomat = FALSE;
 
   unit_list_iterate(ptile->units, punit) {
-    if (unit_has_type_flag(punit, F_DIPLOMAT)) pcity->ai.has_diplomat = TRUE;
+    if (unit_has_type_flag(punit, F_DIPLOMAT)) pcity->ai->has_diplomat = TRUE;
     if (unit_has_type_flag(punit, F_PIKEMEN)) pikemen = TRUE;
   } unit_list_iterate_end;
 
@@ -500,20 +525,33 @@ static unsigned int assess_danger(struct city *pcity)
       /* Although enemy units will not be in our cities,
        * we might still consider allies to be dangerous,
        * so dist can be 0. */
-      bool igwall = unit_really_ignores_citywalls(punit);
 
       if (unit_has_type_flag(punit, F_PARATROOPERS)) {
         paramove = unit_type(punit)->paratroopers_range;
       }
 
-      if ((is_ground_unit(punit) && vulnerability != 0)
-          || (is_ground_units_transport(punit))) {
+      if (is_ground_unit(punit) && vulnerability != 0) {
         if (dist <= move_rate * 3 || dist <= paramove + move_rate) {
           urgency++;
         }
         if (dist <= move_rate || dist <= paramove + move_rate) {
-          pcity->ai.grave_danger++;
+          pcity->ai->grave_danger++;
         }
+      } else {
+        unit_class_iterate(punitclass) {
+          if (punitclass->move_type == LAND_MOVING
+              && can_unit_type_transport(unit_type(punit), punitclass)) {
+            /* It can transport some land moving units! */
+
+            if (dist <= move_rate * 3 || dist <= paramove + move_rate) {
+              urgency++;
+            }
+            if (dist <= move_rate || dist <= paramove + move_rate) {
+              pcity->ai->grave_danger++;
+            }
+            break;
+          }
+        } unit_class_iterate_end;
       }
       if (paramove > 0 && can_unit_paradrop(punit)) {
         move_rate += paramove; /* gross simplification */
@@ -530,73 +568,71 @@ static unsigned int assess_danger(struct city *pcity)
       }
 
       if (unit_has_type_flag(punit, F_DIPLOMAT) && (dist <= 2 * move_rate)) {
-	pcity->ai.diplomat_threat = TRUE;
+	pcity->ai->diplomat_threat = TRUE;
       }
 
       vulnerability *= vulnerability; /* positive feedback */
 
-      if (!igwall) {
-        danger[1] += vulnerability * move_rate / MAX(dist, 1); /* walls */
-      } else if (is_sailing_unit(punit)) {
-        danger[2] += vulnerability * move_rate / MAX(dist, 1); /* coastal */
-      } else if (is_air_unit(punit) && !unit_has_type_flag(punit, F_NUCLEAR)) {
-        danger[3] += vulnerability * move_rate / MAX(dist, 1); /* SAM */
-      }
-      if (unit_has_type_flag(punit, F_MISSILE)) {
-        /* SDI */
-        danger[4] += vulnerability * move_rate / MAX(move_rate, dist);
-      }
-      if (!unit_has_type_flag(punit, F_NUCLEAR)) {
-        /* only SDI helps against NUCLEAR */
-        vulnerability = dangerfunct(vulnerability, move_rate, dist);
-        danger[0] += vulnerability;
-        if (igwall) {
-          igwall_threat += vulnerability;
+      if (unit_has_type_flag(punit, F_NUCLEAR)) {
+        defender = ai_find_source_building(pcity, EFT_NUKE_PROOF,
+                                           unit_class(punit), MOVETYPE_LAST);
+        if (defender != B_LAST) {
+          danger_reduced[defender] += vulnerability * move_rate / MAX(dist, 1);
         }
+      } else if (!unit_has_type_flag(punit, F_IGWALL)) {
+        defender = ai_find_source_building(pcity, EFT_DEFEND_BONUS,
+                                           unit_class(punit), MOVETYPE_LAST);
+        if (defender != B_LAST) {
+          danger_reduced[defender] += vulnerability * move_rate / MAX(dist, 1);
+        }
+      } else {
+        igwall_threat += vulnerability;
       }
+
+      vulnerability = dangerfunct(vulnerability, move_rate, dist);
+      total_danger += vulnerability;
+
     } unit_list_iterate_end;
   } players_iterate_end;
 
   if (igwall_threat == 0) {
-    pcity->ai.wallvalue = 90;
+    pcity->ai->wallvalue = 90;
+  } else if (total_danger) {
+    pcity->ai->wallvalue = ((total_danger * 9 - igwall_threat * 8) 
+                           * 10 / total_danger);
   } else {
-    pcity->ai.wallvalue = (danger[0] * 9 - igwall_threat * 8) 
-                           * 10 / (danger[0]);
+    /* No danger.
+     * This is half of the wallvalue of what danger 1 would produce. */
+    pcity->ai->wallvalue = 5;
   }
 
-  /* Watch out for integer overflows */
-  for (i = 0; i < 5; i++) {
-    if (danger[i] < 0 || danger[i] > 1<<24) {
-      /* I hope never to see this! */
-      freelog(LOG_ERROR, "Dangerous danger[%d] (%d) in %s.  Beware of "
-              "overflow.", i, danger[i], city_name(pcity));
-      danger[i] = danger[i]>>2; /* reduce danger of overflow */
-    }
-  }
-
-  if (pcity->ai.grave_danger != 0) {
+  if (pcity->ai->grave_danger != 0) {
     /* really, REALLY urgent to defend */
     urgency += 10;
   }
 
   /* HACK: This needs changing if multiple improvements provide
    * this effect. */
-  defender = ai_find_source_building(pplayer, EFT_DEFEND_BONUS);
+  /* FIXME: Accept only buildings helping unit classes we actually use.
+   *        Now we consider any land mover helper suitable. */
+  defense = assess_defense_igwall(pcity);
 
-  if (defender != B_LAST) {
-    ai_reevaluate_building(pcity, &pcity->ai.building_want[defender],
-	urgency, danger[1], assess_defense_igwall(pcity));
+  for (i = 0; i < B_LAST; i++) {
+    if (danger_reduced[i] > 0) {
+      ai_reevaluate_building(pcity, &pcity->ai->building_want[i],
+                             urgency, danger_reduced[i], defense);
+    }
   }
 
   if (ai_handicap(pplayer, H_DANGER)
-      && danger[0] == 0) {
+      && total_danger == 0) {
     /* Has to have some danger
      * Otherwise grave_danger will be ignored. */
-    pcity->ai.danger = 1;
+    pcity->ai->danger = 1;
   } else {
-    pcity->ai.danger = danger[0];
+    pcity->ai->danger = total_danger;
   }
-  pcity->ai.urgency = urgency;
+  pcity->ai->urgency = urgency;
 
   TIMING_LOG(AIT_DANGER, TIMER_STOP);
 
@@ -616,9 +652,9 @@ int ai_unit_defence_desirability(const struct unit_type *punittype)
   /* Sea and helicopters often have their firepower set to 1 when
    * defending. We can't have such units as defenders. */
   if (!utype_has_flag(punittype, F_BADCITYDEFENDER)
-      && punittype->move_type != HELI_MOVING) {
+      && !utype_has_flag(punittype, F_HELICOPTER)) {
     /* Sea units get 1 firepower in Pearl Harbour,
-     * and helicopters very bad against air units */
+     * and helicopters very bad against fighters */
     desire *= punittype->firepower;
   }
   desire *= defense;
@@ -684,10 +720,10 @@ static bool process_defender_want(struct player *pplayer, struct city *pcity,
   int best_unit_cost = 1;
 
   memset(tech_desire, 0, sizeof(tech_desire));
-  
+
   simple_ai_unit_type_iterate(punittype) {
     int desire; /* How much we want the unit? */
-    int move_type = punittype->move_type;
+    int move_type = utype_move_type(punittype);
 
     /* Only consider proper defenders - otherwise waste CPU and
      * bump tech want needlessly. */
@@ -711,14 +747,14 @@ static bool process_defender_want(struct player *pplayer, struct city *pcity,
     desire /= POWER_DIVIDER/2; /* Good enough, no rounding errors. */
     desire *= desire;
 
-    if (can_build_unit(pcity, punittype)) {
+    if (can_city_build_unit_now(pcity, punittype)) {
       /* We can build the unit now... */
 
-      int build_cost = unit_build_shield_cost(punittype);
+      int build_cost = utype_build_shield_cost(punittype);
       int limit_cost = pcity->shield_stock + 40;
 
       if (walls && move_type == LAND_MOVING) {
-	desire *= pcity->ai.wallvalue;
+	desire *= pcity->ai->wallvalue;
 	/* TODO: More use of POWER_FACTOR ! */
 	desire /= POWER_FACTOR;
       }
@@ -735,28 +771,27 @@ static bool process_defender_want(struct player *pplayer, struct city *pcity,
 	best_unit_type = punittype;
 	best_unit_cost = build_cost;
       }
-    } else if (can_eventually_build_unit(pcity, punittype)) {
+    } else if (can_city_build_unit_later(pcity, punittype)) {
       /* We first need to develop the tech required by the unit... */
 
       /* Cost (shield equivalent) of gaining these techs. */
       /* FIXME? Katvrr advises that this should be weighted more heavily in
        * big danger. */
       int tech_cost = total_bulbs_required_for_goal(pplayer,
-					punittype->tech_requirement) / 4
-	/ city_list_size(pplayer->cities);
+			advance_number(punittype->require_advance)) / 4
+		      / city_list_size(pplayer->cities);
         
       /* Contrary to the above, we don't care if walls are actually built 
        * - we're looking into the future now. */
       if (move_type == LAND_MOVING) {
-	desire *= pcity->ai.wallvalue;
+	desire *= pcity->ai->wallvalue;
 	desire /= POWER_FACTOR;
       }
 
       /* Yes, there's some similarity with kill_desire(). */
       /* TODO: Explain what shield cost has to do with tech want. */
-      tech_desire[punittype->index] = (desire * danger /
-				       (unit_build_shield_cost(punittype)
-					+ tech_cost));
+      tech_desire[utype_index(punittype)] =
+        (desire * danger / (utype_build_shield_cost(punittype) + tech_cost));
     }
   } simple_ai_unit_type_iterate_end;
 
@@ -765,8 +800,8 @@ static bool process_defender_want(struct player *pplayer, struct city *pcity,
   }
 
   if (best_unit_type) {
-    if (!walls && best_unit_type->move_type == LAND_MOVING) {
-      best *= pcity->ai.wallvalue;
+    if (!walls && utype_move_type(best_unit_type) == LAND_MOVING) {
+      best *= pcity->ai->wallvalue;
       best /= POWER_FACTOR;
     }
   } else {
@@ -779,14 +814,14 @@ static bool process_defender_want(struct player *pplayer, struct city *pcity,
 
   /* Update tech_want for appropriate techs for units we want to build. */
   simple_ai_unit_type_iterate(punittype) {
-    if (tech_desire[punittype->index] > 0) {
-      Tech_type_id tech_req = punittype->tech_requirement;
+    if (tech_desire[utype_index(punittype)] > 0) {
       /* TODO: Document or fix the algorithm below. I have no idea why
        * it is written this way, and the results seem strange to me. - Per */
-      int desire = tech_desire[punittype->index] * best_unit_cost / best;
+      int desire = tech_desire[utype_index(punittype)] * best_unit_cost / best;
 
-      pplayer->ai.tech_want[tech_req] += desire;
-      TECH_LOG(LOG_DEBUG, pplayer, tech_req,
+      pplayer->ai_data.tech_want[advance_index(punittype->require_advance)]
+        += desire;
+      TECH_LOG(LOG_DEBUG, pplayer, punittype->require_advance,
                "+ %d for %s to defend %s",
                desire,
                utype_rule_name(punittype),
@@ -798,7 +833,7 @@ static bool process_defender_want(struct player *pplayer, struct city *pcity,
     return FALSE;
   }
 
-  choice->choice = best_unit_type->index;
+  choice->value.utype = best_unit_type;
   choice->want = danger;
   choice->type = CT_DEFENDER;
   return TRUE;
@@ -832,19 +867,18 @@ static void process_attacker_want(struct city *pcity,
 {
   struct player *pplayer = city_owner(pcity);
   /* The enemy city.  acity == NULL means stray enemy unit */
-  struct city *acity = tile_get_city(ptile);
+  struct city *acity = tile_city(ptile);
   bool shore = is_ocean_near_tile(pcity->tile);
-  int orig_move_type = utype_by_number(best_choice->choice)->move_type;
+  int orig_move_type = utype_move_type(best_choice->value.utype);
   int victim_count = 1;
   int needferry = 0;
-  bool unhap = ai_assess_military_unhappiness(pcity,
-                                              government_of_player(pplayer));
+  bool unhap = ai_assess_military_unhappiness(pcity);
 
   assert(orig_move_type == SEA_MOVING || orig_move_type == LAND_MOVING);
 
   if (orig_move_type == LAND_MOVING && !boat && boattype) {
     /* cost of ferry */
-    needferry = unit_build_shield_cost(boattype);
+    needferry = utype_build_shield_cost(boattype);
   }
   
   if (!is_stack_vulnerable(ptile)) {
@@ -854,21 +888,14 @@ static void process_attacker_want(struct city *pcity,
   }
 
   simple_ai_unit_type_iterate(punittype) {
-    Tech_type_id tech_req = punittype->tech_requirement;
-    int move_type = punittype->move_type;
-    int tech_dist;
-    
-    if (tech_req != A_LAST) {
-      tech_dist = num_unknown_techs_for_goal(pplayer, tech_req);
-    } else {
-      tech_dist = 0;
-    }
-    
+    Tech_type_id tech_req = advance_number(punittype->require_advance);
+    int tech_dist = num_unknown_techs_for_goal(pplayer, tech_req);
+    int move_type = utype_move_type(punittype);
+
     if ((move_type == LAND_MOVING || (move_type == SEA_MOVING && shore))
-        && tech_req != A_LAST
         && (tech_dist > 0 
             || U_NOT_OBSOLETED == punittype->obsoleted_by
-            || !can_build_unit_direct(pcity, 
+            || !can_city_build_unit_direct(pcity, 
                                       punittype->obsoleted_by))
         && punittype->attack_strength > 0 /* or we'll get SIGFPE */
         && move_type == orig_move_type) {
@@ -877,25 +904,26 @@ static void process_attacker_want(struct city *pcity,
       int move_time;
       int vuln;
 
-      /* TODO: check for the right _type_ of building. */
       int will_be_veteran
-	= (ai_find_source_building(pplayer, EFT_VETERAN_BUILD) != B_LAST);
+	= (ai_find_source_building(pcity, EFT_VETERAN_BUILD,
+                                   utype_class(punittype), MOVETYPE_LAST) != B_LAST);
       /* Cost (shield equivalent) of gaining these techs. */
       /* FIXME? Katvrr advises that this should be weighted more heavily in big
        * danger. */
-      int tech_cost = total_bulbs_required_for_goal(pplayer,
-                        punittype->tech_requirement) / 4
+      int tech_cost = total_bulbs_required_for_goal(pplayer, tech_req) / 4
                       / city_list_size(pplayer->cities);
       int move_rate = punittype->move_rate;
       int bcost_balanced = build_cost_balanced(punittype);
       /* See description of kill_desire() for info about this variables. */
-      int bcost = unit_build_shield_cost(punittype);
+      int bcost = utype_build_shield_cost(punittype);
       int attack = unittype_att_rating(punittype, will_be_veteran,
                                        SINGLE_MOVE,
                                        punittype->hp);
       
       /* Take into account reinforcements strength */
-      if (acity) attack += acity->ai.attack;
+      if (acity) {
+        attack += acity->ai->attack;
+      }
 
       if (attack == 0) {
        /* Yes, it can happen that a military unit has attack=1,
@@ -937,23 +965,25 @@ static void process_attacker_want(struct city *pcity,
       if (move_type != LAND_MOVING && vuln == 0) {
         desire = 0;
         
-      } else if ((move_type == LAND_MOVING || move_type == HELI_MOVING)
-                 && acity && acity->ai.invasion == 2) {
+      } else if (uclass_has_flag(utype_class(punittype), UCF_CAN_OCCUPY_CITY)
+                 && acity
+                 && acity->ai->invasion.attack > 0
+                 && acity->ai->invasion.occupy == 0) {
         desire = bcost * SHIELD_WEIGHTING;
 
       } else {
         if (!acity) {
           desire = kill_desire(value, attack, bcost, vuln, victim_count);
         } else {
-          int city_attack = acity->ai.attack * acity->ai.attack;
+          int city_attack = acity->ai->attack * acity->ai->attack;
 
           /* See aiunit.c:find_something_to_kill() for comments. */
           
-          desire = kill_desire(value, attack, (bcost + acity->ai.bcost), vuln,
+          desire = kill_desire(value, attack, (bcost + acity->ai->bcost), vuln,
                                victim_count);
           
-          if (value * city_attack > acity->ai.bcost * vuln) {
-            desire -= kill_desire(value, city_attack, acity->ai.bcost, vuln,
+          if (value * city_attack > acity->ai->bcost * vuln) {
+            desire -= kill_desire(value, city_attack, acity->ai->bcost, vuln,
                                   victim_count);
           }
         }
@@ -972,43 +1002,49 @@ static void process_attacker_want(struct city *pcity,
       if (want > 0) {
         if (tech_dist > 0) {
           /* This is a future unit, tell the scientist how much we need it */
-          pplayer->ai.tech_want[tech_req] += want;
-          TECH_LOG(LOG_DEBUG, pplayer, tech_req,
+          pplayer->ai_data.tech_want[advance_index(punittype->require_advance)]
+            += want;
+          TECH_LOG(LOG_DEBUG, pplayer, punittype->require_advance,
                    "+ %d for %s vs %s(%d,%d)",
                    want,
                    utype_rule_name(punittype),
                    (acity ? city_name(acity) : utype_rule_name(victim_unit_type)),
                    TILE_XY(ptile));
         } else if (want > best_choice->want) {
-          if (can_build_unit(pcity, punittype)) {
+          struct impr_type *impr_req = punittype->need_improvement;
+
+          if (can_city_build_unit_now(pcity, punittype)) {
             /* This is a real unit and we really want it */
 
             CITY_LOG(LOG_DEBUG, pcity, "overriding %s(%d) with %s(%d)"
                      " [attack=%d,value=%d,move_time=%d,vuln=%d,bcost=%d]",
-                     utype_rule_name(utype_by_number(best_choice->choice)),
+                     utype_rule_name(best_choice->value.utype),
 		     best_choice->want,
                      utype_rule_name(punittype),
                      want,
                      attack, value, move_time, vuln, bcost);
 
-            best_choice->choice = punittype->index;
+            best_choice->value.utype = punittype;
             best_choice->want = want;
             best_choice->type = CT_ATTACKER;
-          } else if (can_build_improvement(pcity,
-					   punittype->impr_requirement)) {
+          } else if (NULL == impr_req) {
+            CITY_LOG(LOG_DEBUG, pcity, "cannot build unit %s",
+                     utype_rule_name(punittype));
+          } else if (can_city_build_improvement_now(pcity, impr_req)) {
 	    /* Building this unit requires a specific type of improvement.
 	     * So we build this improvement instead.  This may not be the
 	     * best behavior. */
-            Impr_type_id id = punittype->impr_requirement;
-
-            CITY_LOG(LOG_DEBUG, pcity, "building %s to build %s",
-                     improvement_rule_name(id),
+            CITY_LOG(LOG_DEBUG, pcity, "building %s to build unit %s",
+                     improvement_rule_name(impr_req),
                      utype_rule_name(punittype));
-            best_choice->choice = id;
+            best_choice->value.building = impr_req;
             best_choice->want = want;
             best_choice->type = CT_BUILDING;
           } else {
 	    /* This should never happen? */
+            CITY_LOG(LOG_DEBUG, pcity, "cannot build %s or unit %s",
+                     improvement_rule_name(impr_req),
+                     utype_rule_name(punittype));
 	  }
         }
       }
@@ -1051,35 +1087,36 @@ static void kill_something_with(struct player *pplayer, struct city *pcity,
   struct ai_choice best_choice;
 
   init_choice(&best_choice);
-  best_choice.choice = unit_type(myunit)->index;
+  best_choice.value.utype = unit_type(myunit);
   best_choice.type = CT_ATTACKER;
   best_choice.want = choice->want;
 
-  assert(is_military_unit(myunit) && !is_air_unit(myunit));
+  assert(is_military_unit(myunit) && !utype_fuel(unit_type(myunit)));
 
-  if (pcity->ai.danger != 0 && assess_defense(pcity) == 0) {
+  if (pcity->ai->danger != 0 && assess_defense(pcity) == 0) {
     /* Defence comes first! */
     return;
   }
 
   if (!is_ground_unit(myunit) && !is_sailing_unit(myunit)) {
-    freelog(LOG_ERROR, "ERROR: Attempting to deal with non-trivial"
-            " unit_type in kill_something_with");
+    freelog(LOG_ERROR, "kill_something_with()"
+            " attempting to deal with non-trivial unit_type");
     return;
   }
 
   best_choice.want = find_something_to_kill(pplayer, myunit, &ptile);
 
-  acity = tile_get_city(ptile);
+  acity = tile_city(ptile);
 
   if (myunit->id != 0) {
-    freelog(LOG_ERROR, "ERROR: Non-virtual unit in kill_something_with!");
+    freelog(LOG_ERROR, "kill_something_with()"
+            " non-virtual unit!");
     return;
   }
   
   attack = unit_att_rating(myunit);
   if (acity) {
-    attack += acity->ai.attack;
+    attack += acity->ai->attack;
   }
   attack *= attack;
   
@@ -1115,7 +1152,7 @@ static void kill_something_with(struct player *pplayer, struct city *pcity,
           boattype = get_role_unit(L_FERRYBOAT, 0);
         }
       }
-      assert(SEA_MOVING == boattype->move_type);
+      assert(LAND_MOVING != utype_move_type(boattype));
 
       go_by_boat = !(WARMAP_COST(ptile) <= (MIN(6, move_rate) * THRESHOLD)
                      && goto_is_sane(myunit, acity->tile, TRUE));
@@ -1127,13 +1164,13 @@ static void kill_something_with(struct player *pplayer, struct city *pcity,
     move_time = turns_to_enemy_city(unit_type(myunit), acity, move_rate, 
                                     go_by_boat, ferryboat, boattype);
 
-    def_type = ai_choose_defender_versus(acity, unit_type(myunit));
+    def_type = ai_choose_defender_versus(acity, myunit);
     def_owner = city_owner(acity);
     if (move_time > 1 && def_type) {
       def_vet = do_make_unit_veteran(acity, def_type);
       vuln = unittype_def_rating_sq(unit_type(myunit), def_type, city_owner(acity),
                                     ptile, FALSE, def_vet);
-      benefit = unit_build_shield_cost(def_type);
+      benefit = utype_build_shield_cost(def_type);
     } else {
       vuln = 0;
       benefit = 0;
@@ -1146,13 +1183,13 @@ static void kill_something_with(struct player *pplayer, struct city *pcity,
                                      ptile, FALSE, pdef->veteran);
       if (vuln < m) {
         vuln = m;
-        benefit = unit_build_shield_cost(unit_type(pdef));
+        benefit = unit_build_shield_cost(pdef);
         def_vet = pdef->veteran;
         def_type = unit_type(pdef); 
 	def_owner = unit_owner(pdef);
       }
     }
-    if (COULD_OCCUPY(myunit) || TEST_BIT(acity->ai.invasion, 0)) {
+    if (COULD_OCCUPY(myunit) || acity->ai->invasion.occupy > 0) {
       /* bonus for getting the city */
       benefit += 40;
     }
@@ -1166,7 +1203,7 @@ static void kill_something_with(struct player *pplayer, struct city *pcity,
       return;
     }
 
-    benefit = unit_build_shield_cost(unit_type(pdef));
+    benefit = unit_build_shield_cost(pdef);
     go_by_boat = FALSE;
 
     def_type = unit_type(pdef);
@@ -1191,10 +1228,11 @@ static void kill_something_with(struct player *pplayer, struct city *pcity,
   if (best_choice.want > choice->want) {
     /* We want attacker more than what we have selected before */
     copy_if_better_choice(&best_choice, choice);
-    CITY_LOG(LOG_DEBUG, pcity, "ksw: %s has chosen attacker, %s, want=%d",
+    CITY_LOG(LOG_DEBUG, pcity, "kill_something_with()"
+	     " %s has chosen attacker, %s, want=%d",
 	     city_name(pcity),
-	     utype_rule_name(utype_by_number(choice->choice)),
-	     choice->want);
+	     utype_rule_name(best_choice.value.utype),
+	     best_choice.want);
 
     if (go_by_boat && !ferryboat) { /* need a new ferry */
       /* We might need a new boat even if there are boats free,
@@ -1202,13 +1240,13 @@ static void kill_something_with(struct player *pplayer, struct city *pcity,
       assert(is_ground_unit(myunit));
       ai_choose_role_unit(pplayer, pcity, choice, CT_ATTACKER,
                           L_FERRYBOAT, choice->want, TRUE);
-      if (SEA_MOVING == utype_by_number(choice->choice)->move_type) {
+      if (SEA_MOVING == utype_move_type(choice->value.utype)) {
         struct ai_data *ai = ai_data_get(pplayer);
 
-        freelog(LOG_DEBUG,
-                "%s has chosen attacker ferry, %s, want=%d, %d of %d free",
+        freelog(LOG_DEBUG, "kill_something_with()"
+                " %s has chosen attacker ferry, %s, want=%d, %d of %d free",
                 city_name(pcity),
-                utype_rule_name(utype_by_number(choice->choice)),
+                utype_rule_name(choice->value.utype),
 		choice->want,
                 ai->stats.available_boats, ai->stats.boats);
       } /* else can not build ferries yet */
@@ -1238,7 +1276,7 @@ static void ai_unit_consider_bodyguard(struct city *pcity,
 
     if (want > choice->want) {
       choice->want = want;
-      choice->choice = punittype->index;
+      choice->value.utype = punittype;
       choice->type = CT_DEFENDER;
     }
   }
@@ -1257,23 +1295,23 @@ static void adjust_ai_unit_choice(struct city *pcity,
                                   struct ai_choice *choice)
 {
   enum unit_move_type move_type;
-  struct player *pplayer = city_owner(pcity);
   Impr_type_id id;
 
   /* Sanity */
   if (!is_unit_choice_type(choice->type)
-      || utype_has_flag(utype_by_number(choice->choice), F_NONMIL)
-      || do_make_unit_veteran(pcity, utype_by_number(choice->choice))) {
+      || utype_has_flag(choice->value.utype, F_CIVILIAN)
+      || do_make_unit_veteran(pcity, choice->value.utype)) {
     return;
   }
 
-  move_type = utype_by_number(choice->choice)->move_type;
+  move_type = utype_move_type(choice->value.utype);
 
-  /* TODO: separate checks based on other requirements (e.g., unit class) 
-   *  N.B.: have to check that we haven't already built the building --mck */
-  if ((id = ai_find_source_building(pplayer, EFT_VETERAN_BUILD)) != B_LAST
-       && !city_got_building(pcity, id)) {
-    choice->choice = id;
+  /*  N.B.: have to check that we haven't already built the building --mck */
+  if ((id = ai_find_source_building(pcity, EFT_VETERAN_BUILD,
+                                    utype_class(choice->value.utype),
+                                    MOVETYPE_LAST)) != B_LAST
+       && !city_has_building(pcity, improvement_by_number(id))) {
+    choice->value.building = improvement_by_number(id);
     choice->type = CT_BUILDING;
   }
 }
@@ -1301,30 +1339,31 @@ void military_advisor_choose_build(struct player *pplayer, struct city *pcity,
   /* It has to be AFTER assess_danger thanks to wallvalue. */
   our_def = assess_defense_quadratic(pcity); 
 
-  if (pcity->id == ai->wonder_city && pcity->ai.grave_danger == 0) {
+  if (pcity->id == ai->wonder_city && pcity->ai->grave_danger == 0) {
     return; /* Other cities can build our defenders, thank you! */
   }
 
   ai_choose_diplomat_defensive(pplayer, pcity, choice, our_def);
 
   /* Otherwise no need to defend yet */
-  if (pcity->ai.danger != 0) { 
+  if (pcity->ai->danger != 0) {
+    struct impr_type *pimprove;
     int num_defenders = unit_list_size(ptile->units);
     int wall_id, danger;
 
     /* First determine the danger.  It is measured in percents of our 
      * defensive strength, capped at 200 + urgency */
-    if (pcity->ai.danger >= our_def) {
+    if (pcity->ai->danger >= our_def) {
       if (urgency == 0) {
         /* don't waste money */
         danger = 100;
       } else if (our_def == 0) {
         danger = 200 + urgency;
       } else {
-        danger = MIN(200, 100 * pcity->ai.danger / our_def) + urgency;
+        danger = MIN(200, 100 * pcity->ai->danger / our_def) + urgency;
       }
     } else { 
-      danger = 100 * pcity->ai.danger / our_def;
+      danger = 100 * pcity->ai->danger / our_def;
     }
     if (pcity->surplus[O_SHIELD] <= 0 && our_def != 0) {
       /* Won't be able to support anything */
@@ -1334,26 +1373,27 @@ void military_advisor_choose_build(struct player *pplayer, struct city *pcity,
     CITY_LOG(LOG_DEBUG, pcity, "m_a_c_d urgency=%d danger=%d num_def=%d "
              "our_def=%d", urgency, danger, num_defenders, our_def);
 
-    /* FIXME: 1. Will tend to build walls beofre coastal irrespectfully what
+    /* FIXME: 1. Will tend to build walls before coastal irrespectfully what
      * type of danger we are facing */
     /* We will build walls if we can and want and (have "enough" defenders or
      * can just buy the walls straight away) */
 
     /* HACK: This needs changing if multiple improvements provide
      * this effect. */
-    wall_id = ai_find_source_building(pplayer, EFT_DEFEND_BONUS);
+    wall_id = ai_find_source_building(pcity, EFT_DEFEND_BONUS, NULL, LAND_MOVING);
+    pimprove = improvement_by_number(wall_id);
 
     if (wall_id != B_LAST
-	&& pcity->ai.building_want[wall_id] != 0 && our_def != 0 
-        && can_build_improvement(pcity, wall_id)
+	&& pcity->ai->building_want[wall_id] != 0 && our_def != 0 
+        && can_city_build_improvement_now(pcity, pimprove)
         && (danger < 101 || num_defenders > 1
-            || (pcity->ai.grave_danger == 0 
-                && pplayer->economic.gold > impr_buy_gold_cost(wall_id, pcity->shield_stock)))
+            || (pcity->ai->grave_danger == 0 
+                && pplayer->economic.gold > impr_buy_gold_cost(pimprove, pcity->shield_stock)))
         && ai_fuzzy(pplayer, TRUE)) {
       /* NB: great wall is under domestic */
-      choice->choice = wall_id;
+      choice->value.building = pimprove;
       /* building_want is hacked by assess_danger */
-      choice->want = pcity->ai.building_want[wall_id];
+      choice->want = pcity->ai->building_want[wall_id];
       if (urgency == 0 && choice->want > 100) {
         choice->want = 100;
       }
@@ -1365,7 +1405,7 @@ void military_advisor_choose_build(struct player *pplayer, struct city *pcity,
       if (process_defender_want(pplayer, pcity, danger, choice)) {
         /* Potential defender found */
         if (urgency == 0
-            && utype_by_number(choice->choice)->defense_strength == 1) {
+            && choice->value.utype->defense_strength == 1) {
           /* FIXME: check other reqs (unit class?) */
           if (get_city_bonus(pcity, EFT_HP_REGEN) > 0) {
             /* unlikely */
@@ -1377,7 +1417,7 @@ void military_advisor_choose_build(struct player *pplayer, struct city *pcity,
           choice->want = danger;
         }
         CITY_LOG(LOG_DEBUG, pcity, "m_a_c_d wants %s with desire %d",
-                 utype_rule_name(utype_by_number(choice->choice)),
+                 utype_rule_name(choice->value.utype),
                  choice->want);
       } else {
         CITY_LOG(LOG_DEBUG, pcity, "m_a_c_d cannot select defender");
@@ -1404,7 +1444,7 @@ void military_advisor_choose_build(struct player *pplayer, struct city *pcity,
   /* If we are in severe danger, don't consider attackers. This is probably
      too general. In many cases we will want to buy attackers to counterattack.
      -- Per */
-  if (choice->want > 100 && pcity->ai.grave_danger > 0) {
+  if (choice->want > 100 && pcity->ai->grave_danger > 0) {
     CITY_LOG(LOGLEVEL_BUILD, pcity, "severe danger (want %d), force defender",
              choice->want);
     return;
@@ -1453,12 +1493,9 @@ void military_advisor_choose_build(struct player *pplayer, struct city *pcity,
 
   if (choice->want <= 0) {
     CITY_LOG(LOGLEVEL_BUILD, pcity, "military advisor has no advice");
-  } else if (is_unit_choice_type(choice->type)) {
-    CITY_LOG(LOGLEVEL_BUILD, pcity, "military advisor choice: %s (want %d)",
-             utype_rule_name(utype_by_number(choice->choice)),
-             choice->want);
   } else {
     CITY_LOG(LOGLEVEL_BUILD, pcity, "military advisor choice: %s (want %d)",
-             improvement_rule_name(choice->choice), choice->want);
+             ai_choice_rule_name(choice),
+             choice->want);
   }
 }
