@@ -113,10 +113,12 @@ void cityresult_fill(struct player *pplayer,
                      struct ai_data *ai,
                      struct cityresult *result)
 {
-  struct city *pcity = tile_get_city(result->tile);
+  struct city *pcity = tile_city(result->tile);
+  struct government *curr_govt = government_of_player(pplayer);
+  struct player *saved_owner = NULL;
+  struct tile *saved_claimer = NULL;
   int sum = 0;
   bool virtual_city = FALSE;
-  struct government *curr_govt = government_of_player(pplayer);
   bool handicap = ai_handicap(pplayer, H_MAP);
 
   pplayer->government = ai->goal.govt.gov;
@@ -132,35 +134,39 @@ void cityresult_fill(struct player *pplayer,
 
   if (!pcity) {
     pcity = create_city_virtual(pplayer, result->tile, "Virtuaville");
+    saved_owner = tile_owner(result->tile);
+    saved_claimer = tile_claimer(result->tile);
+    tile_set_owner(result->tile, pplayer, result->tile); /* temporarily */
+    city_choose_build_default(pcity);  /* ?? */
     virtual_city = TRUE;
   }
 
-  city_map_checked_iterate(result->tile, i, j, ptile) {
+  city_tile_iterate_cxy(result->tile, ptile, i, j) {
     int reserved = citymap_read(ptile);
-    bool city_center = is_city_center(i, j);
+    bool city_center = (result->tile == ptile); /*is_city_center()*/
 
     if (reserved < 0
         || (handicap && !map_is_known(ptile, pplayer))
-        || ptile->worked != NULL) {
+        || NULL != tile_worked(ptile)) {
       /* Tile is reserved or we can't see it */
       result->citymap[i][j].shield = 0;
       result->citymap[i][j].trade = 0;
       result->citymap[i][j].food = 0;
       sum = 0;
-    } else if (cachemap[ptile->index].sum <= 0 || city_center) {
+    } else if (cachemap[tile_index(ptile)].sum <= 0 || city_center) {
       /* We cannot read city center from cache */
 
       /* Food */
       result->citymap[i][j].food
-	= base_city_get_output_tile(i, j, pcity, FALSE, O_FOOD);
+	= city_tile_output(pcity, ptile, FALSE, O_FOOD);
 
       /* Shields */
       result->citymap[i][j].shield
-	= base_city_get_output_tile(i, j, pcity, FALSE, O_SHIELD);
+	= city_tile_output(pcity, ptile, FALSE, O_SHIELD);
 
       /* Trade */
       result->citymap[i][j].trade
-	= base_city_get_output_tile(i, j, pcity, FALSE, O_TRADE);
+	= city_tile_output(pcity, ptile, FALSE, O_TRADE);
 
       sum = result->citymap[i][j].food * ai->food_priority
             + result->citymap[i][j].trade * ai->science_priority
@@ -175,16 +181,16 @@ void cityresult_fill(struct player *pplayer,
       if (!city_center && virtual_city) {
         /* real cities and any city center will give us spossibly
          * skewed results */
-        cachemap[ptile->index].sum = sum;
-        cachemap[ptile->index].trade = result->citymap[i][j].trade;
-        cachemap[ptile->index].shield = result->citymap[i][j].shield;
-        cachemap[ptile->index].food = result->citymap[i][j].food;
+        cachemap[tile_index(ptile)].sum = sum;
+        cachemap[tile_index(ptile)].trade = result->citymap[i][j].trade;
+        cachemap[tile_index(ptile)].shield = result->citymap[i][j].shield;
+        cachemap[tile_index(ptile)].food = result->citymap[i][j].food;
       }
     } else {
-      sum = cachemap[ptile->index].sum;
-      result->citymap[i][j].shield = cachemap[ptile->index].shield;
-      result->citymap[i][j].trade = cachemap[ptile->index].trade;
-      result->citymap[i][j].food = cachemap[ptile->index].food;
+      sum = cachemap[tile_index(ptile)].sum;
+      result->citymap[i][j].shield = cachemap[tile_index(ptile)].shield;
+      result->citymap[i][j].trade = cachemap[tile_index(ptile)].trade;
+      result->citymap[i][j].food = cachemap[tile_index(ptile)].food;
     }
     result->citymap[i][j].reserved = reserved;
 
@@ -210,7 +216,7 @@ void cityresult_fill(struct player *pplayer,
        * of the area and the emphasis placed on space for growth. */
       result->remaining += sum / GROWTH_POTENTIAL_DEEMPHASIS;
     }
-  } city_map_checked_iterate_end;
+  } city_tile_iterate_cxy_end;
 
   if (virtual_city) {
     /* Baseline is a size one city (city center + best extra tile). */
@@ -261,7 +267,8 @@ void cityresult_fill(struct player *pplayer,
 
   pplayer->government = curr_govt;
   if (virtual_city) {
-    remove_city_virtual(pcity);
+    destroy_city_virtual(pcity);
+    tile_set_owner(result->tile, saved_owner, saved_claimer);
   }
 
   assert(result->city_center >= 0);
@@ -296,7 +303,7 @@ static int defense_bonus(struct cityresult *result, struct ai_data *ai)
 {
   /* Defense modification (as tie breaker mostly) */
   int defense_bonus = 
-    10 + tile_get_terrain(result->tile)->defense_bonus / 10;
+    10 + tile_terrain(result->tile)->defense_bonus / 10;
   if (tile_has_special(result->tile, S_RIVER)) {
     defense_bonus +=
         (defense_bonus * terrain_control.river_defense_bonus) / 100;
@@ -326,7 +333,7 @@ static int naval_bonus(struct cityresult *result, struct ai_data *ai)
 void print_cityresult(struct player *pplayer, struct cityresult *cr,
                       struct ai_data *ai)
 {
-  freelog(LOG_NORMAL, "Result=(%d, %d)\nReservations:\n"
+  freelog(LOG_TEST, "Result=(%d, %d)\nReservations:\n"
           "     %4d %4d %4d   \n"
           "%4d %4d %4d %4d %4d\n"
           "%4d %4d %4d %4d %4d\n"
@@ -344,7 +351,7 @@ void print_cityresult(struct player *pplayer, struct cityresult *cr,
           cr->citymap[1][4].reserved, cr->citymap[2][4].reserved,
           cr->citymap[3][4].reserved);
 #define M(a,b) cr->citymap[a][b].food, cr->citymap[a][b].shield, cr->citymap[a][b].trade
-  freelog(LOG_NORMAL, "Tiles (food/shield/trade):\n"
+  freelog(LOG_TEST, "Tiles (food/shield/trade):\n"
           "      %d-%d-%d %d-%d-%d %d-%d-%d\n"
           "%d-%d-%d %d-%d-%d %d-%d-%d %d-%d-%d %d-%d-%d\n"
           "%d-%d-%d %d-%d-%d %d-%d-%d %d-%d-%d %d-%d-%d\n"
@@ -354,7 +361,7 @@ void print_cityresult(struct player *pplayer, struct cityresult *cr,
           M(0,2), M(1,2), M(2,2), M(3,2), M(4,2), M(0,3), M(1,3), M(2,3),
           M(3,3), M(4,3), M(1,4), M(2,4), M(3,4));
 #undef M
-  freelog(LOG_NORMAL, "city center %d + best other(%d, %d) %d - corr %d "
+  freelog(LOG_TEST, "city center %d + best other(%d, %d) %d - corr %d "
           "- waste %d\n"
           "+ remaining %d + defense bonus %d + naval bonus %d = %d (%d)", 
           cr->city_center, cr->other_tile->x, cr->other_tile->y,
@@ -362,10 +369,10 @@ void print_cityresult(struct player *pplayer, struct cityresult *cr,
           cr->corruption, cr->waste, cr->remaining, defense_bonus(cr, ai), 
           naval_bonus(cr, ai), cr->total, cr->result);
   if (food_starvation(cr)) {
-    freelog(LOG_NORMAL, " ** FOOD STARVATION **");
+    freelog(LOG_TEST, " ** FOOD STARVATION **");
   }
   if (shield_starvation(cr)) {
-    freelog(LOG_NORMAL, " ** RESOURCE STARVATION **");
+    freelog(LOG_TEST, " ** RESOURCE STARVATION **");
   }
 }
 
@@ -378,7 +385,7 @@ static void city_desirability(struct player *pplayer, struct ai_data *ai,
                               struct unit *punit, struct tile *ptile,
                               struct cityresult *result)
 {  
-  struct city *pcity = tile_get_city(ptile);
+  struct city *pcity = tile_city(ptile);
 
   assert(punit && ai && pplayer && result);
 
@@ -402,7 +409,7 @@ static void city_desirability(struct player *pplayer, struct ai_data *ai,
     return;
   }
 
-  if (pcity && (pcity->size + unit_pop_value(unit_type(punit))
+  if (pcity && (pcity->size + unit_pop_value(punit)
 		> game.info.add_to_size_limit)) {
     /* Can't exceed population limit. */
     return;
@@ -469,19 +476,18 @@ static bool settler_map_iterate(struct pf_parameter *parameter,
   int best_turn = 0; /* Which turn we found the best fit */
   struct player *pplayer = unit_owner(punit);
   struct ai_data *ai = ai_data_get(pplayer);
-  struct pf_map *map;
+  struct pf_map *pfm;
   bool found = FALSE; /* The return value */
 
-  map = pf_create_map(parameter);
-  pf_iterator(map, pos) {
+  pfm = pf_map_new(parameter);
+  pf_map_iterate_move_costs(pfm, ptile, move_cost, FALSE) {
     int turns;
-    struct tile *ptile = pos.tile;
 
-    if (is_ocean(ptile->terrain)) {
+    if (is_ocean_tile(ptile)) {
       continue; /* This can happen if there is a ferry near shore. */
     }
     if (boat_cost == 0
-        && ptile->continent != punit->tile->continent) {
+        && tile_continent(ptile) != tile_continent(punit->tile)) {
       /* We have an accidential land bridge. Ignore it. It will in all
        * likelihood go away next turn, or even in a few nanoseconds. */
       continue;
@@ -497,7 +503,7 @@ static bool settler_map_iterate(struct pf_parameter *parameter,
     }
 
     /* Calculate worth */
-    city_desirability(pplayer, ai, punit, pos.tile, &result);
+    city_desirability(pplayer, ai, punit, ptile, &result);
 
     /* Check if actually found something */
     if (result.total == 0) {
@@ -505,14 +511,14 @@ static bool settler_map_iterate(struct pf_parameter *parameter,
     }
 
     /* This algorithm punishes long treks */
-    turns = pos.total_MC / parameter->move_rate;
+    turns = move_cost / parameter->move_rate;
     result.result = amortize(result.total, PERFECTION * turns);
 
     /* Reduce want by settler cost. Easier than amortize, but still
      * weeds out very small wants. ie we create a threshold here. */
     /* We also penalise here for using a boat (either virtual or real)
      * it's crude but what isn't? */
-    result.result -= unit_build_shield_cost(unit_type(punit)) + boat_cost;
+    result.result -= unit_build_shield_cost(punit) + boat_cost;
 
     /* Find best spot */
     if (result.result > best->result) {
@@ -529,9 +535,9 @@ static bool settler_map_iterate(struct pf_parameter *parameter,
         && best_turn < turns /*+ game.info.min_dist_bw_cities*/) {
       break;
     }
-  } pf_iterator_end;
+  } pf_map_iterate_positions_end;
 
-  pf_destroy_map(map);
+  pf_map_destroy(pfm);
 
   assert(!found || 0 <= best->result);
   return found;
@@ -554,8 +560,9 @@ void find_best_city_placement(struct unit *punit, struct cityresult *best,
   struct pf_parameter parameter;
   struct player *pplayer = unit_owner(punit);
   struct unit *ferry = NULL;
+  struct unit_class *ferry_class = NULL;
 
-  assert(pplayer->ai.control);
+  assert(pplayer->ai_data.control);
   /* Only virtual units may use virtual boats: */
   assert(0 == punit->id || !use_virt_boat);
 
@@ -584,7 +591,7 @@ void find_best_city_placement(struct unit *punit, struct cityresult *best,
 
   if (ferry 
       || (use_virt_boat && is_ocean_near_tile(punit->tile) 
-          && tile_get_city(punit->tile))) {
+          && tile_city(punit->tile))) {
     if (!ferry) {
       /* No boat?  Get a virtual one! */
       struct unit_type *boattype
@@ -592,13 +599,14 @@ void find_best_city_placement(struct unit *punit, struct cityresult *best,
 
       if (boattype == NULL) {
         /* Sea travel not possible yet. Bump tech want for ferries. */
-        struct unit_type *boattype = get_role_unit(L_FERRYBOAT, 0);
+        boattype = get_role_unit(L_FERRYBOAT, 0);
 
-        if (boattype != NULL) {
-          Tech_type_id tech_req = boattype->tech_requirement;
-
-          pplayer->ai.tech_want[tech_req] += FERRY_TECH_WANT;
-          TECH_LOG(LOG_DEBUG, pplayer, tech_req, "+ %d for %s to ferry settler",
+        if (NULL != boattype
+         && A_NEVER != boattype->require_advance) {
+          pplayer->ai_data.tech_want[advance_index(boattype->require_advance)]
+            += FERRY_TECH_WANT;
+          TECH_LOG(LOG_DEBUG, pplayer, boattype->require_advance,
+                   "+ %d for %s to ferry settler",
                    FERRY_TECH_WANT,
                    utype_rule_name(boattype));
         }
@@ -608,7 +616,9 @@ void find_best_city_placement(struct unit *punit, struct cityresult *best,
       ferry->tile = punit->tile;
     }
 
-    assert(LAND_MOVING != unit_type(ferry)->move_type);
+    ferry_class = unit_class(ferry);
+
+    assert(ferry_class->ai.sea_move != MOVE_NONE);
     pft_fill_unit_overlap_param(&parameter, ferry);
     parameter.get_TB = no_fights_or_unknown;
 
@@ -617,7 +627,7 @@ void find_best_city_placement(struct unit *punit, struct cityresult *best,
      * Building a new boat is like a war against a weaker enemy -- 
      * good for the economy. (c) Bush family */
     if (settler_map_iterate(&parameter, punit, best,
-			    unit_build_shield_cost(unit_type(ferry)))) {
+			    unit_build_shield_cost(ferry))) {
       best->overseas = TRUE;
       best->virt_boat = (ferry->id == 0);
     }
