@@ -20,14 +20,17 @@
 #include <stdlib.h>
 #include <string.h>
 
+/* utility */
 #include "capability.h"
 #include "fcintl.h"
 #include "log.h"
 #include "mem.h"
 #include "registry.h"
 #include "shared.h"
+#include "string_vector.h"
 #include "support.h"
 
+/* client */
 #include "audio_none.h"
 #ifdef AUDIO_SDL
 #include "audio_sdl.h"
@@ -46,37 +49,46 @@ static int num_plugins_used = 0;
 static int selected_plugin = -1;
 
 /**********************************************************************
-  Returns a static, NULL-terminated list of all sound plugins
+  Returns a static string vector of all sound plugins
   available on the system.  This function is unfortunately similar to
   audio_get_all_plugin_names().
 ***********************************************************************/
-const char **get_soundplugin_list(void)
+const struct strvec *get_soundplugin_list(void)
 {
-  static const char* plugin_list[MAX_NUM_PLUGINS + 1];
-  int i;
-  
-  for (i = 0; i < num_plugins_used; i++) {
-    plugin_list[i] = plugins[i].name;
+  static struct strvec *plugin_list = NULL;
+
+  if (NULL == plugin_list) {
+    int i;
+
+    plugin_list = strvec_new();
+    strvec_reserve(plugin_list, num_plugins_used);
+    for (i = 0; i < num_plugins_used; i++) {
+      strvec_set(plugin_list, i, plugins[i].name);
+    }
   }
-  assert(i <= MAX_NUM_PLUGINS);
-  plugin_list[i] = NULL;
 
   return plugin_list;
 }
 
 /**********************************************************************
-  Returns a static list of soundsets available on the system by
+  Returns a static string vector of soundsets available on the system by
   searching all data directories for files matching SNDSPEC_SUFFIX.
   The list is NULL-terminated.
 ***********************************************************************/
-const char **get_soundset_list(void)
+const struct strvec *get_soundset_list(void)
 {
-  static const char **audio_list = NULL;
+  static struct strvec *audio_list = NULL;
 
-  if (!audio_list) {
-    /* Note: this means you must restart the client after installing a new
-       soundset. */
-    audio_list = (const char **)datafilelist(SNDSPEC_SUFFIX);
+  if (NULL == audio_list) {
+    char **list, **file;
+
+    audio_list = strvec_new();
+    list = datafilelist(SNDSPEC_SUFFIX);
+    for (file = list; NULL != *file; file++) {
+      strvec_append(audio_list, *file);
+      free(*file);
+    }
+    free(list);
   }
 
   return audio_list;
@@ -122,7 +134,7 @@ bool audio_select_plugin(const char *const name)
   }
 
   if (!plugins[i].init()) {
-    freelog(LOG_ERROR, _("Plugin %s found but can't be initialized."), name);
+    freelog(LOG_ERROR, "Plugin %s found, but can't be initialized.", name);
     return FALSE;
   }
 
@@ -171,7 +183,7 @@ static const char *soundspec_fullname(const char *soundset_name)
     return NULL;
   }
 
-  freelog(LOG_ERROR, _("Couldn't find soundset \"%s\" trying \"%s\"."),
+  freelog(LOG_ERROR, "Couldn't find soundset \"%s\", trying \"%s\".",
 	  soundset_name, soundset_default);
   return soundspec_fullname(soundset_default);
 }
@@ -188,31 +200,31 @@ void audio_real_init(const char *const spec_name,
 
   if (strcmp(prefered_plugin_name, "none") == 0) {
     /* We explicitly choose none plugin, silently skip the code below */
-    freelog(LOG_VERBOSE, "Proceeding with sound support disabled");
+    freelog(LOG_VERBOSE, "Proceeding with sound support disabled.");
     tagfile = NULL;
     return;
   }
   if (num_plugins_used == 1) {
     /* We only have the dummy plugin, skip the code but issue an advertise */
     freelog(LOG_NORMAL, _("No real audio plugin present."));
-    freelog(LOG_NORMAL, _("Proceeding with sound support disabled"));
+    freelog(LOG_NORMAL, _("Proceeding with sound support disabled."));
     freelog(LOG_NORMAL, _("For sound support, install SDL_mixer"));
     freelog(LOG_NORMAL, "http://www.libsdl.org/projects/SDL_mixer/index.html");
     tagfile = NULL;
     return;
   }
   if (!spec_name) {
-    freelog(LOG_FATAL, _("No sound spec-file given!"));
+    freelog(LOG_FATAL, "No sound spec-file given!");
     exit(EXIT_FAILURE);
   }
   freelog(LOG_VERBOSE, "Initializing sound using %s...", spec_name);
   filename = soundspec_fullname(spec_name);
   if (!filename) {
-    freelog(LOG_ERROR, _("Cannot find sound spec-file \"%s\"."), spec_name);
+    freelog(LOG_ERROR, "Cannot find sound spec-file \"%s\".", spec_name);
     freelog(LOG_NORMAL, _("To get sound you need to download a sound set!"));
     freelog(LOG_NORMAL, _("Get sound sets from <%s>."),
 	    "ftp://ftp.freeciv.org/freeciv/contrib/audio/soundsets");
-    freelog(LOG_NORMAL, _("Proceeding with sound support disabled"));
+    freelog(LOG_NORMAL, _("Proceeding with sound support disabled."));
     tagfile = NULL;
     return;
   }
@@ -244,18 +256,17 @@ void audio_real_init(const char *const spec_name,
 
   if (prefered_plugin_name[0] != '\0') {
     if (!audio_select_plugin(prefered_plugin_name))
-      freelog(LOG_NORMAL, _("Proceeding with sound support disabled"));
+      freelog(LOG_NORMAL, _("Proceeding with sound support disabled."));
     return;
   }
 
 #ifdef AUDIO_SDL
   if (audio_select_plugin("sdl")) return; 
 #endif
+  freelog(LOG_NORMAL, _("No real audio subsystem managed to initialize!"));
   freelog(LOG_NORMAL,
-    _("No real audio subsystem managed to initialize!"));
-  freelog(LOG_NORMAL,
-    _("Perhaps there is some misconfiguration or bad permissions"));
-  freelog(LOG_NORMAL, _("Proceeding with sound support disabled"));
+          _("Perhaps there is some misconfiguration or bad permissions."));
+  freelog(LOG_NORMAL, _("Proceeding with sound support disabled."));
 }
 
 /**************************************************************************
@@ -277,7 +288,7 @@ static bool audio_play_tag(const char *tag, bool repeat)
     } else {
       fullpath = datafilename(soundfile);
       if (!fullpath) {
-	freelog(LOG_ERROR, _("Cannot find audio file %s"), soundfile);
+	freelog(LOG_ERROR, "Cannot find audio file %s", soundfile);
       }
     }
   }
