@@ -32,8 +32,8 @@
 #include "log.h"
 
 /* client */
-#include "civclient.h"
-#include "clinet.h"
+#include "client_main.h"
+#include "clinet.h"		/* connect_to_server() */
 #include "packhand.h"
 #include "servers.h"
 
@@ -54,7 +54,7 @@
 
 #include "connectdlg.h"
 
-static struct server_list *pServer_list = NULL;
+static const struct server_list *pServer_list = NULL;
 static struct server_scan *pServer_scan = NULL; 
     
 static struct ADVANCED_DLG *pMeta_Severs = NULL;
@@ -96,7 +96,7 @@ static int connect_callback(struct widget *pWidget)
     if (connect_to_server(user_name, server_host, server_port,
                           errbuf, sizeof(errbuf)) != -1) {
     } else {
-      append_output_window(errbuf);
+      output_window_append(ftc_any, errbuf);
       real_update_meswin_dialog();
   
       /* button up */
@@ -148,7 +148,7 @@ static int sellect_meta_severs_callback(struct widget *pWidget)
     struct server *pServer = (struct server *)pWidget->data.ptr;
         
     sz_strlcpy(server_host, pServer->host);
-    sscanf(pServer->port, "%d", &server_port);
+    server_port = pServer->port;
     
     exit_meta_severs_dlg_callback(NULL);
   }
@@ -161,7 +161,7 @@ static int sellect_meta_severs_callback(struct widget *pWidget)
 static void server_scan_error(struct server_scan *scan,
 			      const char *message)
 {
-  append_output_window(message);
+  output_window_append(ftc_client, message);
   freelog(LOG_NORMAL, "%s", message);
 
   switch (server_scan_get_type(scan)) {
@@ -185,9 +185,9 @@ static void server_scan_error(struct server_scan *scan,
   same functionality for LAN server dettection.
   WARING !: for LAN scan use "finish_lanserver_scan()" to free server list.
 **************************************************************************/
-static struct server_list *sdl_create_server_list(bool lan)
+static const struct server_list *sdl_create_server_list(bool lan)
 {
-  struct server_list *server_list = NULL;
+  const struct server_list *server_list = NULL;
   int i;
     
   if (lan) {
@@ -203,7 +203,8 @@ static struct server_list *sdl_create_server_list(bool lan)
   SDL_Delay(5000);
     
   for (i = 0; i < 100; i++) {
-    server_list = server_scan_get_servers(pServer_scan);
+    server_scan_poll(pServer_scan);
+    server_list = server_scan_get_list(pServer_scan);
     if (server_list) {
       break;
     }
@@ -276,9 +277,9 @@ void popup_connection_dialog(bool lan_scan)
   
   if(!pServer_list) {
     if (lan_scan) {
-      append_output_window(_("No LAN servers found")); 
+      output_window_append(ftc_client, _("No LAN servers found")); 
     } else {
-      append_output_window(_("No public servers found")); 
+      output_window_append(ftc_client, _("No public servers found")); 
     }        
     real_update_meswin_dialog();
     set_client_page(PAGE_NETWORK);
@@ -312,7 +313,9 @@ void popup_connection_dialog(bool lan_scan)
   /* servers */
   server_list_iterate(pServer_list, pServer) {
 
-    my_snprintf(cBuf, sizeof(cBuf), "%s Port %s Ver: %s %s %s %s\n%s",
+    /* TRANS: "host.example.com Port 5556 Ver: 2.2.0 Running Players 3\n
+     * [server message]" */
+    my_snprintf(cBuf, sizeof(cBuf), _("%s Port %d Ver: %s %s %s %d\n%s"),
     	pServer->host, pServer->port, pServer->version, _(pServer->state),
     		_("Players"), pServer->nplayers, pServer->message);
 
@@ -340,10 +343,10 @@ void popup_connection_dialog(bool lan_scan)
 
   if(!count) {
     if (lan_scan) {
-      append_output_window(_("No LAN servers found")); 
+      output_window_append(ftc_client, _("No LAN servers found")); 
     } else {
-      append_output_window(_("No public servers found")); 
-    }        
+      output_window_append(ftc_client, _("No public servers found"));
+    }
     real_update_meswin_dialog();
     set_client_page(PAGE_NETWORK);
     return;
@@ -743,7 +746,7 @@ static int send_passwd_callback(struct widget *pWidget)
     
     flush_dirty();
     
-    send_packet_authentication_reply(&aconnection, &reply);
+    send_packet_authentication_reply(&client.conn, &reply);
   }
   return -1;
 }
@@ -1101,7 +1104,7 @@ void handle_authentication_req(enum authentication_type type, char *message)
       struct packet_authentication_reply reply;
 
       sz_strlcpy(reply.password, password);
-      send_packet_authentication_reply(&aconnection, &reply);
+      send_packet_authentication_reply(&client.conn, &reply);
       return;
     } else {
       popup_user_passwd_dialog(message);
