@@ -96,6 +96,10 @@ enum GUI_ID {
   ID_OPTIONS_MAP_CITY_NAMES_CHECKBOX,
   ID_OPTIONS_MAP_CITY_PROD_LABEL,
   ID_OPTIONS_MAP_CITY_PROD_CHECKBOX,
+  ID_OPTIONS_MAP_CITY_OUTPUT_CHECKBOX,
+  ID_OPTIONS_MAP_CITY_OUTPUT_LABEL,
+  ID_OPTIONS_MAP_CITY_TRADE_ROUTES_CHECKBOX,
+  ID_OPTIONS_MAP_CITY_TRADE_ROUTES_LABEL,
   ID_OPTIONS_MAP_BORDERS_CHECKBOX,
   ID_OPTIONS_MAP_BORDERS_LABEL,
   ID_OPTIONS_MAP_CITY_CIV3_TEXT_STYLE_LABEL,
@@ -124,17 +128,19 @@ enum GUI_ID {
   ID_OPTIONS_MAP_TERRAIN_FOG_LABEL,
   ID_OPTIONS_MAP_TERRAIN_FOG_CHECKBOX,
   ID_OPTIONS_WORKLIST_BUTTON,
+  ID_OPTIONS_SAVE_GAME_BUTTON,
   ID_OPTIONS_DISC_BUTTON,
   ID_OPTIONS_BACK_BUTTON,
   ID_OPTIONS_EXIT_BUTTON,
   ID_UNITS_WINDOW,
   ID_TOGGLE_UNITS_WINDOW_BUTTON,
   ID_MINI_MAP_WINDOW,
+  ID_TOGGLE_MINIMAP_MODE,
   ID_TOGGLE_MAP_WINDOW_BUTTON,
   ID_UNIT_ORDER_BUILD_CITY,
   ID_UNIT_ORDER_BUILD_WONDER,
   ID_UNIT_ORDER_ROAD,
-  ID_UNIT_ORDER_TRADEROUTE,
+  ID_UNIT_ORDER_TRADE_ROUTE,
   ID_UNIT_ORDER_IRRIGATE,
   ID_UNIT_ORDER_MINE,
   ID_UNIT_ORDER_TRANSFORM,
@@ -147,11 +153,15 @@ enum GUI_ID {
   ID_UNIT_ORDER_SENTRY,
   ID_UNIT_ORDER_PILLAGE,
   ID_UNIT_ORDER_HOMECITY,
+  ID_UNIT_ORDER_UNLOAD_TRANSPORTER,
+  ID_UNIT_ORDER_LOAD,
   ID_UNIT_ORDER_UNLOAD,
   ID_UNIT_ORDER_WAKEUP_OTHERS,
-  ID_UNIT_ORDER_AUTOMATION,
+  ID_UNIT_ORDER_AUTO_SETTLER,
   ID_UNIT_ORDER_AUTO_EXPLORE,
-  ID_UNIT_ORDER_CONNECT,
+  ID_UNIT_ORDER_CONNECT_ROAD,
+  ID_UNIT_ORDER_CONNECT_RAILROAD,
+  ID_UNIT_ORDER_CONNECT_IRRIGATE,
   ID_UNIT_ORDER_PATROL,
   ID_UNIT_ORDER_GOTO,
   ID_UNIT_ORDER_GOTO_CITY,
@@ -246,6 +256,7 @@ enum GUI_ID {
   ID_PILLAGE_DLG_EXIT_BUTTON,
   ID_CARAVAN_DLG_WINDOW,
   ID_CARAVAN_DLG_EXIT_BUTTON,
+  ID_DIPLOMAT_DLG_WINDOW,
   ID_INCITE_DLG_WINDOW,
   ID_INCITE_DLG_EXIT_BUTTON,
   ID_CONNECT_DLG_WINDOW,
