@@ -36,11 +36,12 @@
 #include "game.h"
 #include "goto.h"
 #include "government.h"
+#include "movement.h"
 #include "unitlist.h"
 
 /* client */
 #include "citydlg_common.h"
-#include "civclient.h"
+#include "client_main.h"
 #include "climisc.h"
 #include "overview_common.h"
 #include "pages_g.h"
@@ -152,9 +153,12 @@ void queue_flush(void)
        * the flush code would simply stop working.  But this means the
        * below message may be repeated many times. */
       freelog(LOG_ERROR,
-	      _("The SDL event buffer is full; you may see drawing errors\n"
-		"as a result.  If you see this message often, please\n"
-		"report it at %s. Thank you."), BUG_URL);
+              _("The SDL event buffer is full;"
+                " you may see drawing errors as a result."));
+      freelog(LOG_ERROR,
+              /* TRANS: No full stop after the URL, could cause confusion. */
+              _("Please report this message at %s"),
+              BUG_URL);
     }
   }
 }
@@ -308,42 +312,41 @@ void set_indicator_icons(struct sprite *bulb, struct sprite *sol,
   pBuf = get_revolution_widget();
   set_new_icon2_theme(pBuf, adj_surf(GET_SURF(gov)), TRUE);    
   
-  if (game.player_ptr) {
-    my_snprintf(cBuf, sizeof(cBuf), "%s (%s)\n%s", _("Revolution"), "Shift+R",
-                                    government_name_for_player(game.player_ptr));
+  if (NULL != client.conn.playing) {
+    my_snprintf(cBuf, sizeof(cBuf), "%s (%s)\n%s", _("Revolution"), "Ctrl+Shift+R",
+                                    government_name_for_player(client.conn.playing));
   } else {
-    my_snprintf(cBuf, sizeof(cBuf), "%s (%s)\n%s", _("Revolution"), "Shift+R",
-                                    "None");
+    my_snprintf(cBuf, sizeof(cBuf), "%s (%s)\n%s", _("Revolution"), "Ctrl+Shift+R",
+                                    _("None"));
   }        
   copy_chars_to_string16(pBuf->string16, cBuf);
       
   widget_redraw(pBuf);
   widget_mark_dirty(pBuf);
 
-  
   pBuf = get_tax_rates_widget();
   if(!pBuf->theme) {
     set_new_icon2_theme(pBuf, get_tax_surface(O_GOLD), TRUE);
   }
-  
+
   pBuf = get_research_widget();
-  
-  if (!game.player_ptr) {
+
+  if (NULL == client.conn.playing) {
     /* TRANS: Research report action */
     my_snprintf(cBuf, sizeof(cBuf), "%s (%s)\n%s (%d/%d)", _("Research"), "F6",
-                                    "None", 0, 0);
-  } else if (get_player_research(game.player_ptr)->researching != A_UNSET) {  
+                                    _("None"), 0, 0);
+  } else if (A_UNSET != get_player_research(client.conn.playing)->researching) {
     /* TRANS: Research report action */
     my_snprintf(cBuf, sizeof(cBuf), "%s (%s)\n%s (%d/%d)", _("Research"), "F6",
-	      	advance_name_researching(game.player_ptr),
-	      	get_player_research(game.player_ptr)->bulbs_researched,
-  		total_bulbs_required(game.player_ptr));
+		advance_name_researching(client.conn.playing),
+		get_player_research(client.conn.playing)->bulbs_researched,
+		total_bulbs_required(client.conn.playing));
   } else {
     /* TRANS: Research report action */
     my_snprintf(cBuf, sizeof(cBuf), "%s (%s)\n%s (%d/%d)", _("Research"), "F6",
-	      	advance_name_researching(game.player_ptr),
-	      	get_player_research(game.player_ptr)->bulbs_researched,
-  		0);
+		advance_name_researching(client.conn.playing),
+		get_player_research(client.conn.playing)->bulbs_researched,
+		0);
   }
 
   copy_chars_to_string16(pBuf->string16, cBuf);
@@ -406,26 +409,26 @@ void update_info_label(void)
   pText->fgcol = *get_game_colorRGB(COLOR_THEME_MAPVIEW_INFO_TEXT);
   pText->bgcol = (SDL_Color) {0, 0, 0, 0};
 
-  if (game.player_ptr) {
+  if (NULL != client.conn.playing) {
 #ifdef SMALL_SCREEN
     my_snprintf(buffer, sizeof(buffer),
                 _("%s Population: %s  Year: %s  "
                   "Gold %d "),
-                nation_adjective_for_player(game.player_ptr),
-                population_to_text(civ_population(game.player_ptr)),
+                nation_adjective_for_player(client.conn.playing),
+                population_to_text(civ_population(client.conn.playing)),
                 textyear(game.info.year),
-                game.player_ptr->economic.gold);
+                client.conn.playing->economic.gold);
 #else
     my_snprintf(buffer, sizeof(buffer),
                 _("%s Population: %s  Year: %s  "
                   "Gold %d Tax: %d Lux: %d Sci: %d "),
-                nation_adjective_for_player(game.player_ptr),
-                population_to_text(civ_population(game.player_ptr)),
+                nation_adjective_for_player(client.conn.playing),
+                population_to_text(civ_population(client.conn.playing)),
                 textyear(game.info.year),
-                game.player_ptr->economic.gold,
-                game.player_ptr->economic.tax,
-                game.player_ptr->economic.luxury,
-                game.player_ptr->economic.science);
+                client.conn.playing->economic.gold,
+                client.conn.playing->economic.tax,
+                client.conn.playing->economic.luxury,
+                client.conn.playing->economic.science);
 #endif
     /* convert to unistr and create text surface */
     copy_chars_to_string16(pText, buffer);
@@ -481,202 +484,6 @@ static int fucus_units_info_callback(struct widget *pWidget)
   return -1;
 }
 
-/****************************************************************************
-  Return a (static) string with tile name describing terrain and specials.
-  Same as tile_get_info_text() from client/text.c, but with an additional
-  line break
-
-  Examples:
-    "Hills"
-    "Hills (Coals)"
-    "Hills (Coals)
-     [Pollution]"
-****************************************************************************/
-static const char *gui_sdl_tile_get_info_text(const struct tile *ptile)
-{
-  static char s[256];
-  bool first;
-
-  sz_strlcpy(s, terrain_name_translation(ptile->terrain));
-  if (tile_has_special(ptile, S_RIVER)) {
-    sz_strlcat(s, "/");
-    sz_strlcat(s, special_name_translation(S_RIVER));
-  }
-
-  if (tile_resource_is_valid(ptile)) {
-    cat_snprintf(s, sizeof(s), " (%s)",
-		 resource_name_translation(ptile->resource));
-  }
-
-  first = TRUE;
-  if (tile_has_special(ptile, S_POLLUTION)) {
-    if (first) {
-      first = FALSE;
-      sz_strlcat(s, "\n[");
-    } else {
-      sz_strlcat(s, "/");
-    }
-    sz_strlcat(s, special_name_translation(S_POLLUTION));
-  }
-  if (tile_has_special(ptile, S_FALLOUT)) {
-    if (first) {
-      first = FALSE;
-      sz_strlcat(s, "\n[");
-    } else {
-      sz_strlcat(s, "/");
-    }
-    sz_strlcat(s, special_name_translation(S_FALLOUT));
-  }
-  if (!first) {
-    sz_strlcat(s, "]");
-  }
-
-  return s;
-}
-
-/****************************************************************************
-  Return the text body for the unit info shown in the info panel.
-  Same as get_unit_info_label_text2() from client/text.c, but calls
-  gui_sdl_tile_get_info_text() instead.
-
-  FIXME: this should be renamed.
-****************************************************************************/
-static const char *gui_sdl_get_unit_info_label_text2(struct unit_list *punits)
-{
-  static struct astring str = ASTRING_INIT;
-  int count;
-
-  astr_clear(&str);
-
-  if (!punits) {
-    return "";
-  }
-
-  count = unit_list_size(punits);
-
-  /* This text should always have the same number of lines.  Otherwise the
-   * GUI widgets may be confused and try to resize themselves. */
-
-  /* Line 1. Goto or activity text. */
-  if (count > 0 && hover_state != HOVER_NONE) {
-    int min, max;
-
-    if (!goto_get_turns(&min, &max)) {
-      /* TRANS: Impossible to reach goto target tile */
-      astr_add_line(&str, Q_("?goto:Unreachable"), min);
-    } else if (min == max) {
-      astr_add_line(&str, _("Turns to target: %d"), max);
-    } else {
-      astr_add_line(&str, _("Turns to target: %d to %d"), min, max);
-    }
-  } else if (count == 1) {
-    astr_add_line(&str, "%s",
-      unit_activity_text(unit_list_get(punits, 0)));
-  } else if (count > 1) {
-    astr_add_line(&str, PL_("%d unit selected",
-          "%d units selected",
-          count),
-      count);
-  } else {
-    astr_add_line(&str, _("No units selected."));
-  }
-
-  /* Lines 2, 3, and 4 vary. */
-  if (count == 1) {
-    struct unit *punit = unit_list_get(punits, 0);
-    struct city *pcity =
-  player_find_city_by_id(unit_owner(punit), punit->homecity);
-    int infracount;
-    bv_special infrastructure =
-      get_tile_infrastructure_set(punit->tile, &infracount);
-
-    astr_add_line(&str, "%s", gui_sdl_tile_get_info_text(punit->tile));
-    if (infracount > 0) {
-      astr_add_line(&str, "%s", get_infrastructure_text(infrastructure));
-    } else {
-      astr_add_line(&str, " ");
-    }
-    if (pcity) {
-      astr_add_line(&str, "%s", city_name(pcity));
-    } else {
-      astr_add_line(&str, " ");
-    }
-  } else if (count > 1) {
-    int mil = 0, nonmil = 0;
-    int types_count[U_LAST], i;
-    struct unit_type *top[3];
-
-    memset(types_count, 0, sizeof(types_count));
-    unit_list_iterate(punits, punit) {
-      if (unit_has_type_flag(punit, F_NONMIL)) {
-  nonmil++;
-      } else {
-  mil++;
-      }
-      types_count[unit_type(punit)->index]++;
-    } unit_list_iterate_end;
-
-    top[0] = top[1] = top[2] = NULL;
-    unit_type_iterate(utype) {
-      if (!top[2]
-    || types_count[top[2]->index] < types_count[utype->index]) {
-  top[2] = utype;
-
-  if (!top[1]
-      || types_count[top[1]->index] < types_count[top[2]->index]) {
-    top[2] = top[1];
-    top[1] = utype;
-
-    if (!top[0]
-        || types_count[top[0]->index] < types_count[utype->index]) {
-      top[1] = top[0];
-      top[0] = utype;
-    }
-  }
-      }
-    } unit_type_iterate_end;
-
-    for (i = 0; i < 3; i++) {
-      if (top[i] && types_count[top[i]->index] > 0) {
-  if (utype_has_flag(top[i], F_NONMIL)) {
-    nonmil -= types_count[top[i]->index];
-  } else {
-    mil -= types_count[top[i]->index];
-  }
-  astr_add_line(&str, "%d: %s",
-          types_count[top[i]->index], utype_name_translation(top[i]));
-      } else {
-  astr_add_line(&str, " ");
-      }
-    }
-
-    if (nonmil > 0 && mil > 0) {
-      astr_add_line(&str, _("Others: %d civil; %d military"), nonmil, mil);
-    } else if (nonmil > 0) {
-      astr_add_line(&str, _("Others: %d civilian"), nonmil);
-    } else if (mil > 0) {
-      astr_add_line(&str, _("Others: %d military"), mil);
-    } else {
-      astr_add_line(&str, " ");
-    }
-  } else {
-    astr_add_line(&str, " ");
-    astr_add_line(&str, " ");
-    astr_add_line(&str, " ");
-  }
-
-  /* Line 5. Debug text. */
-#ifdef DEBUG
-  if (count == 1) {
-    astr_add_line(&str, "(Unit ID %d)", unit_list_get(punits, 0)->id);
-  } else {
-    astr_add_line(&str, " ");
-  }
-#endif
-
-  return str.str;
-}
-
 /**************************************************************************
   Read Function Name :)
   FIXME: should use same method as client/text.c popup_info_text()
@@ -735,7 +542,9 @@ void redraw_unit_info_label(struct unit_list *punitlist)
       change_ptsize16(pStr, adj_font(10));
       pStr->style |= SF_CENTER;
 
-      copy_chars_to_string16(pStr, gui_sdl_get_unit_info_label_text2(punitlist));
+      copy_chars_to_string16(pStr,
+                             get_unit_info_label_text2(punitlist,
+                                                       TILE_LB_RESOURCE_POLL));
       pInfo = create_text_surf_from_str16(pStr);
       
       if (pInfo_Window->size.h > 
@@ -747,18 +556,20 @@ void redraw_unit_info_label(struct unit_list *punitlist)
 	
         if (pInfo_Window->size.h > 
             2 * h + (DEFAULT_UNITS_H + (pInfo_Window->size.h - pInfo_Window->area.h))|| right) {
-	  if (game.info.borders > 0 && !pTile->city) {
+          struct city *pCity = tile_city(pTile);
+
+          if (game.info.borders > 0 && !pCity) {
 	    const char *diplo_nation_plural_adjectives[DS_LAST] =
                         {Q_("?nation:Neutral"), Q_("?nation:Hostile"),
      			"" /* unused, DS_CEASEFIRE*/,
      			Q_("?nation:Peaceful"), Q_("?nation:Friendly"), 
      			Q_("?nation:Mysterious")};
-            if (tile_owner(pTile) == game.player_ptr) {
+            if (tile_owner(pTile) == client.conn.playing) {
               cat_snprintf(buffer, sizeof(buffer), _("\nOur Territory"));
             } else {
 	      if (tile_owner(pTile)) {
-                if (game.player_ptr->diplstates[tile_owner(pTile)->player_no].type==DS_CEASEFIRE){
-		  int turns = game.player_ptr->diplstates[tile_owner(pTile)->player_no].turns_left;
+                if (DS_CEASEFIRE == client.conn.playing->diplstates[player_index(tile_owner(pTile))].type){
+		  int turns = client.conn.playing->diplstates[player_index(tile_owner(pTile))].turns_left;
 		  cat_snprintf(buffer, sizeof(buffer),
 		  	PL_("\n%s territory (%d turn ceasefire)",
 				"\n%s territory (%d turn ceasefire)", turns),
@@ -766,19 +577,19 @@ void redraw_unit_info_label(struct unit_list *punitlist)
                 } else {
 	          cat_snprintf(buffer, sizeof(buffer), _("\nTerritory of the %s %s"),
 		    diplo_nation_plural_adjectives[
-		  	game.player_ptr->diplstates[tile_owner(pTile)->player_no].type],
-		    		nation_plural_for_player(tile_owner(pTile)));
+			client.conn.playing->diplstates[player_index(tile_owner(pTile))].type],
+				nation_plural_for_player(tile_owner(pTile)));
                 }
               } else { /* !tile_owner(pTile) */
                 cat_snprintf(buffer, sizeof(buffer), _("\nUnclaimed territory"));
               }
 	    }
-          } /* game.info.borders > 0 && !pTile->city */
+          } /* game.info.borders > 0 && !pCity */
           
-          if (pTile->city) {
+          if (pCity) {
             /* Look at city owner, not tile owner (the two should be the same, if
              * borders are in use). */
-            struct player *pOwner = city_owner(pTile->city);
+            struct player *pOwner = city_owner(pCity);
             bool citywall;
 /*            bool barrack = FALSE, airport = FALSE, port = FALSE;*/
 	    const char *diplo_city_adjectives[DS_LAST] =
@@ -788,20 +599,20 @@ void redraw_unit_info_label(struct unit_list *punitlist)
 			  
 	    cat_snprintf(buffer, sizeof(buffer),
 			 _("\nCity of %s"),
-			 city_name(pTile->city));
+			 city_name(pCity));
 
-	    citywall = pTile->city->client.walls;
+	    citywall = pCity->client.walls;
                           
 #if 0       
             /* This has hardcoded assumption that EFT_LAND_REGEN is always
              * provided by *building* named *Barracks*. Similar assumptions for
              * other effects. */     
-	    if (pplayers_allied(game.player_ptr, pOwner)) {
-	      barrack = (get_city_bonus(pTile->city, EFT_LAND_REGEN) > 0);
-	      airport = (get_city_bonus(pTile->city, EFT_AIR_VETERAN) > 0);
-	      port = (get_city_bonus(pTile->city, EFT_SEA_VETERAN) > 0);
+	    if (pplayers_allied(client.conn.playing, pOwner)) {
+	      barrack = (get_city_bonus(pCity, EFT_LAND_REGEN) > 0);
+	      airport = (get_city_bonus(pCity, EFT_AIR_VETERAN) > 0);
+	      port = (get_city_bonus(pCity, EFT_SEA_VETERAN) > 0);
 	    }
-	  
+
 	    if (citywall || barrack || airport || port) {
 	      cat_snprintf(buffer, sizeof(buffer), Q_("?blistbegin: with "));
 	      if (barrack) {
@@ -830,11 +641,11 @@ void redraw_unit_info_label(struct unit_list *punitlist)
 	    }
 #endif
 	    
-	    if (pOwner && pOwner != game.player_ptr) {
+	    if (pOwner && pOwner != client.conn.playing) {
               /* TRANS: (<nation>,<diplomatic_state>)" */
               cat_snprintf(buffer, sizeof(buffer), _("\n(%s,%s)"),
 		  nation_adjective_for_player(pOwner),
-		  diplo_city_adjectives[game.player_ptr->
+		  diplo_city_adjectives[client.conn.playing->
 				   diplstates[player_index(pOwner)].type]);
 	    }
 	    
@@ -1002,8 +813,8 @@ void redraw_unit_info_label(struct unit_list *punitlist)
           if (++n > num_w * num_h) {
              set_wflag(pBuf, WF_HIDDEN);
           }
-  
-          if (unit_owner(aunit) == game.player_ptr) {    
+
+          if (unit_owner(aunit) == client.conn.playing) {
             set_wstate(pBuf, FC_WS_NORMAL);
           }
     
@@ -1076,7 +887,7 @@ void redraw_unit_info_label(struct unit_list *punitlist)
 	hide_scrollbar(pInfo_Window->private_data.adv_dlg->pScroll);
       }
 
-      if (game.player_ptr) {
+      if (NULL != client.conn.playing) {
         char buf[256];
         my_snprintf(buf, sizeof(buf), "%s\n%s\n%s",
                     _("End of Turn"), _("Press"), _("Shift+Return"));
@@ -1153,54 +964,6 @@ void update_unit_info_label(struct unit_list *punitlist)
     if(!is_anim_enabled()) {
       enable_focus_animation();
     }
-    if (action_state == CURSOR_ACTION_WAIT) {
-      update_mouse_cursor(CURSOR_WAIT);
-    } else {
-    switch (hover_state) {
-    case HOVER_NONE:
-      if (action_state == CURSOR_ACTION_SELECT) {
-        update_mouse_cursor(CURSOR_SELECT); 
-      } else if (action_state == CURSOR_ACTION_PARATROOPER) {
-        update_mouse_cursor(CURSOR_PARADROP);  
-      } else if (action_state == CURSOR_ACTION_NUKE) {
-        update_mouse_cursor(CURSOR_NUKE);
-      } else {
-        update_mouse_cursor(CURSOR_DEFAULT);
-      }  
-      break;
-    case HOVER_GOTO:
-      if (action_state == CURSOR_ACTION_GOTO) {
-        update_mouse_cursor(CURSOR_GOTO);
-      } else if (action_state == CURSOR_ACTION_DEFAULT) {
-        update_mouse_cursor(CURSOR_DEFAULT);
-      } else if (action_state == CURSOR_ACTION_ATTACK) {
-        update_mouse_cursor(CURSOR_ATTACK); 
-      } else {
-        update_mouse_cursor(CURSOR_INVALID);  
-      }
-      break;
-    case HOVER_PATROL:
-      if (action_state == CURSOR_ACTION_INVALID) {
-        update_mouse_cursor(CURSOR_INVALID);
-      } else {
-        update_mouse_cursor(CURSOR_PATROL);
-      }
-      break;
-    case HOVER_CONNECT:
-      if (action_state == CURSOR_ACTION_INVALID) {
-        update_mouse_cursor(CURSOR_INVALID);
-      } else {
-        update_mouse_cursor(CURSOR_GOTO);
-      }
-      break;
-    case HOVER_NUKE:
-      update_mouse_cursor(CURSOR_NUKE);
-      break;
-    case HOVER_PARADROP:
-      update_mouse_cursor(CURSOR_PARADROP);
-      break;
-    }
-    }
   } else {
     disable_focus_animation();
   }
@@ -1347,7 +1110,7 @@ void tileset_changed(void)
   /* PORTME */
   /* Here you should do any necessary redraws (for instance, the city
    * dialogs usually need to be resized). */
-  
+  popdown_all_game_dialogs();
 }
 
 /* =====================================================================
