@@ -15,11 +15,13 @@
 #include <config.h>
 #endif
 
+/* common & utility */
 #include "fcintl.h"
 #include "game.h"
 #include "support.h"
 
-#include "civclient.h"
+/* client */
+#include "client_main.h"
 #include "climisc.h"
 #include "control.h"
 #include "mapctrl_common.h"
@@ -41,11 +43,12 @@ void update_info_label(void)
 		"Year: %s\n"
 		"Gold %d\n"
 		"Tax: %d Lux: %d Sci: %d"),
-	      population_to_text(civ_population(game.player_ptr)),
-	      textyear(game.info.year), game.player_ptr->economic.gold,
-	      game.player_ptr->economic.tax,
-	      game.player_ptr->economic.luxury,
-	      game.player_ptr->economic.science);
+	      population_to_text(civ_population(client.conn.playing)),
+	      textyear(game.info.year),
+	      client.conn.playing->economic.gold,
+	      client.conn.playing->economic.tax,
+	      client.conn.playing->economic.luxury,
+	      client.conn.playing->economic.science);
 
   /* ... */
 }
@@ -67,6 +70,15 @@ void update_unit_info_label(struct unit_list *punitlist)
   /* PORTME */
 }
 
+/****************************************************************************
+  Update the mouse cursor. Cursor type depends on what user is doing and
+  pointing.
+****************************************************************************/
+void update_mouse_cursor(enum cursor_type new_cursor_type)
+{
+  /* PORTME */
+}
+
 /****************************************************************************
   Update the timeout display.  The timeout is the time until the turn
   ends, in seconds.
