@@ -22,10 +22,10 @@
 
 #include <windows.h>
 #include <windowsx.h>
- 
+
+/* common & utility */ 
 #include "city.h"
 #include "fcintl.h"
-#include "game.h"
 #include "genlist.h"
 #include "government.h"
 #include "map.h"
@@ -36,7 +36,9 @@
 #include "unit.h"
 #include "support.h"
 #include "version.h"
- 
+
+/* client */
+#include "client_main.h"
 #include "climisc.h"
 #include "colors.h"
 #include "gui_stuff.h"
@@ -163,7 +165,7 @@ static LONG APIENTRY HelpdlgWndProc(HWND hWnd,UINT uMsg,
 	    char s[128];
 	    GetWindowText((HWND)lParam,s,sizeof(s));
 	    if (strcmp(s, _("(Never)")) != 0 && strcmp(s, _("None")) != 0
-		&& strcmp(s, advance_name_translation(A_NONE)) != 0)
+		&& strcmp(s, advance_name_translation(advance_by_number(A_NONE))) != 0)
 	      select_help_item_string(s,page_type_from_id(LOWORD(wParam)));
 	    
 	  }
@@ -466,18 +468,18 @@ void create_help_dialog()
 ...
 **************************************************************************/
 static void help_update_improvement(const struct help_item *pitem,
-                                    char *title, int which)
+                                    char *title)
 {
   char buf[64000];
- 
-  create_help_page(HELP_IMPROVEMENT);
- 
-  if (which < game.control.num_impr_types) {
-    struct impr_type *imp = improvement_by_number(which);
-    int i;
-    char req_buf[512];
+  struct impr_type *imp = find_improvement_by_translated_name(title);
 
-    sprintf(buf, "%d", impr_build_shield_cost(which));
+  create_help_page(HELP_IMPROVEMENT);
+
+  if (imp  &&  !is_great_wonder(imp)) {
+    char req_buf[512];
+    int i;
+
+    sprintf(buf, "%d", impr_build_shield_cost(imp));
     SetWindowText(help_ilabel[1], buf);
     sprintf(buf, "%d", imp->upkeep);
     SetWindowText(help_ilabel[3], buf);
@@ -489,9 +491,10 @@ static void help_update_improvement(const struct help_item *pitem,
     i = 0;
     requirement_vector_iterate(&imp->reqs, preq) {
       SetWindowText(help_ilabel[5 + i],
-		    get_req_source_text(&preq->source, req_buf,
+		    universal_name_translation(&preq->source, req_buf,
 		    sizeof(req_buf)));
       i++;
+      break;
     } requirement_vector_iterate_end;
 /*    create_tech_tree(help_improvement_tree, 0, imp->tech_req, 3);*/
   }
@@ -499,9 +502,9 @@ static void help_update_improvement(const struct help_item *pitem,
     SetWindowText(help_ilabel[1], "0");
     SetWindowText(help_ilabel[3], "0");
     SetWindowText(help_ilabel[5], _("(Never)"));
-/*    create_tech_tree(help_improvement_tree, 0, game.control.num_tech_types, 3);*/
+/*    create_tech_tree(help_improvement_tree, 0, advance_count(), 3);*/
   }
-  helptext_building(buf, sizeof(buf), which, pitem->text);
+  helptext_building(buf, sizeof(buf), client.conn.playing, pitem->text, imp);
   set_help_text(buf);
 
 }
@@ -509,21 +512,19 @@ static void help_update_improvement(const struct help_item *pitem,
 ...
 **************************************************************************/
 static void help_update_wonder(const struct help_item *pitem,
-                               char *title, int which)
+                               char *title)
 {
   char buf[64000];
- 
-  create_help_page(HELP_WONDER);
- 
-  if (which < game.control.num_impr_types) {
-    struct impr_type *imp = improvement_by_number(which);
-    int i;
-    char req_buf[512];
+  struct impr_type *imp = find_improvement_by_translated_name(title);
 
-    sprintf(buf, "%d", impr_build_shield_cost(which));
+  create_help_page(HELP_WONDER);
+
+  if (imp  &&  is_great_wonder(imp)) {
+    char req_buf[512];
+    int i;
+
+    sprintf(buf, "%d", impr_build_shield_cost(imp));
     SetWindowText(help_ilabel[1], buf);
-    sprintf(buf, "%d", imp->upkeep);
-    SetWindowText(help_ilabel[3], buf);
 
     /* FIXME: this should show ranges and all the MAX_NUM_REQS reqs. 
      * Currently it's limited to 1 req but this code is partially prepared
@@ -531,11 +532,20 @@ static void help_update_wonder(const struct help_item *pitem,
      * definition. */
     i = 0;
     requirement_vector_iterate(&imp->reqs, preq) {
-      SetWindowText(help_ilabel[5 + i],
-		    get_req_source_text(&preq->source, req_buf,
+      SetWindowText(help_ilabel[3 + i],
+		    universal_name_translation(&preq->source, req_buf,
 		    sizeof(req_buf)));
       i++;
+      break;
     } requirement_vector_iterate_end;
+    if (valid_advance(imp->obsolete_by)) {
+      SetWindowText(help_ilabel[5],
+                    advance_name_for_player(client.conn.playing,
+					    advance_number(imp->obsolete_by)));
+    } else {
+      SetWindowText(help_ilabel[5], _("None"));
+    }
+
 /*    create_tech_tree(help_improvement_tree, 0, imp->tech_req, 3);*/
   }
   else {
@@ -543,10 +553,10 @@ static void help_update_wonder(const struct help_item *pitem,
     SetWindowText(help_ilabel[1], "0");
     SetWindowText(help_ilabel[3], _("(Never)"));
     SetWindowText(help_ilabel[5], _("None"));
-/*    create_tech_tree(help_improvement_tree, 0, game.control.num_tech_types, 3); */
+/*    create_tech_tree(help_improvement_tree, 0, advance_count(), 3); */
   }
  
-  helptext_building(buf, sizeof(buf), which, pitem->text);
+  helptext_building(buf, sizeof(buf), client.conn.playing, pitem->text, imp);
   set_help_text(buf);
 }                            
 
@@ -554,9 +564,10 @@ static void help_update_wonder(const struct help_item *pitem,
 ...
 **************************************************************************/
 static void help_update_terrain(const struct help_item *pitem,
-				char *title, struct terrain *pterrain)
+				char *title)
 {
   char buf[64000];
+  struct terrain *pterrain = find_terrain_by_translated_name(title);
 
   create_help_page(HELP_TERRAIN);
 
@@ -637,7 +648,7 @@ static void help_update_terrain(const struct help_item *pitem,
     SetWindowText(help_tlabel[3][4], buf);
   }
 
-  helptext_terrain(buf, sizeof(buf), pterrain, pitem->text);
+  helptext_terrain(buf, sizeof(buf), client.conn.playing, pitem->text, pterrain);
   set_help_text(buf);
 }
 
@@ -656,12 +667,23 @@ static void help_draw_unit(HDC hdc, struct unit_type *utype)
   
   /* Give tile a background color, based on the type of unit
    * FIXME: make a new set of colors for this.               */
-  switch (utype->move_type) {
-  case LAND_MOVING: bg_color = COLOR_OVERVIEW_LAND;       break;
-  case SEA_MOVING:  bg_color = COLOR_OVERVIEW_OCEAN;      break;
-  case HELI_MOVING: bg_color = COLOR_OVERVIEW_MY_UNIT;    break;
-  case AIR_MOVING:  bg_color = COLOR_OVERVIEW_ENEMY_CITY; break;
-  default:          bg_color = COLOR_OVERVIEW_UNKNOWN;    break;
+  switch (unit_color_type(utype)) {
+  case UNIT_BG_LAND:
+    bg_color = COLOR_OVERVIEW_LAND;
+    break;
+  case UNIT_BG_SEA:
+    bg_color = COLOR_OVERVIEW_OCEAN;
+    break;
+  case UNIT_BG_HP_LOSS:
+  case UNIT_BG_AMPHIBIOUS:
+    bg_color = COLOR_OVERVIEW_MY_UNIT;
+    break;
+  case UNIT_BG_FLYING:
+    bg_color = COLOR_OVERVIEW_ENEMY_CITY;
+    break;
+  default:
+    bg_color = COLOR_OVERVIEW_UNKNOWN;
+    break;
   }
 
   brush = brush_alloc(get_color(tileset, bg_color));
@@ -682,15 +704,17 @@ static void help_draw_unit(HDC hdc, struct unit_type *utype)
 
 **************************************************************************/
 static void help_update_unit_type(const struct help_item *pitem,
-				  char *title, struct unit_type *utype)
+				  char *title)
 {
   char buf[64000];
+  struct unit_type *utype = find_unit_type_by_translated_name(title);
+
   create_help_page(HELP_UNIT);
 
   drawn_unit_type = utype;
 
   if (utype) {
-    sprintf(buf, "%d", unit_build_shield_cost(utype));
+    sprintf(buf, "%d", utype_build_shield_cost(utype));
     SetWindowText(help_ulabel[0][1], buf);
     sprintf(buf, "%d", utype->attack_strength);
     SetWindowText(help_ulabel[0][4], buf);
@@ -705,19 +729,22 @@ static void help_update_unit_type(const struct help_item *pitem,
     SetWindowText(help_ulabel[3][1], helptext_unit_upkeep_str(utype));
     sprintf(buf, "%d", utype->vision_radius_sq);
     SetWindowText(help_ulabel[3][4], buf);
-    if(utype->tech_requirement==A_LAST) {
+    if (A_NEVER == utype->require_advance) {
       SetWindowText(help_ulabel[4][1], _("(Never)"));
     } else {
-      SetWindowText(help_ulabel[4][1], advance_name_translation(utype->tech_requirement));
+      SetWindowText(help_ulabel[4][1],
+                    advance_name_for_player(client.conn.playing,
+				       advance_number(utype->require_advance)));
     }
-    /*    create_tech_tree(help_improvement_tree, 0, utype->tech_requirement, 3);*/
-    if (utype->obsoleted_by == U_NOT_OBSOLETED) {
+    /*    create_tech_tree(help_improvement_tree, 0, advance_number(utype->require_advance), 3);*/
+    if (U_NOT_OBSOLETED == utype->obsoleted_by) {
       SetWindowText(help_ulabel[4][4], _("None"));
     } else {
-      SetWindowText(help_ulabel[4][4], utype_name_translation(utype->obsoleted_by));
+      SetWindowText(help_ulabel[4][4],
+                    utype_name_translation(utype->obsoleted_by));
     }
 
-    helptext_unit(buf, sizeof(buf), utype, pitem->text);
+    helptext_unit(buf, sizeof(buf), client.conn.playing, pitem->text, utype);
     set_help_text(buf);
   }
   else {
@@ -740,95 +767,105 @@ static void help_update_unit_type(const struct help_item *pitem,
 /**************************************************************************
 
 **************************************************************************/
-static void help_update_tech(const struct help_item *pitem, char *title, int i)
+static void help_update_tech(const struct help_item *pitem, char *title)
 {
-  int j;
   struct fcwin_box *hbox;
   char buf[64000];
+  int i;
+  struct advance *padvance = find_advance_by_translated_name(title);
 
   create_help_page(HELP_TECH);
-  if (!is_future_tech(i)) {
+
+  if (padvance  &&  !is_future_tech(i = advance_number(padvance))) {
     /*    
 	  create_tech_tree(GTK_CTREE(help_tree), i, TECH_TREE_DEPTH,
 	  TECH_TREE_EXPANDED_DEPTH, NULL);
     */
-    helptext_tech(buf, sizeof(i), i, pitem->text);
+    helptext_advance(buf, sizeof(buf), client.conn.playing, pitem->text, i);
     wordwrap_string(buf, 68);
     fcwin_box_add_static(helpdlg_page_vbox,buf,0,SS_LEFT,FALSE,FALSE,5);
 
-    impr_type_iterate(j) {
+    improvement_iterate(pimprove) {
       /* FIXME: need a more general mechanism for this, since this
        * helptext needs to be shown in all possible req source types. */
-     requirement_vector_iterate(&improvement_by_number(j)->reqs, req) {
-	if (req->source.type == REQ_NONE) {
+      requirement_vector_iterate(&pimprove->reqs, req) {
+	if (VUT_NONE == req->source.kind) {
 	  break;
-	} else if (req->source.type == REQ_BUILDING
-		   && req->source.value.building == i) {
+	}
+	if (VUT_IMPROVEMENT == req->source.kind
+	 && req->source.value.building == pimprove) {
 	  hbox = fcwin_hbox_new(helpdlg_win, FALSE);
 	  fcwin_box_add_box(helpdlg_page_vbox, hbox, FALSE, FALSE, 5);
 	  fcwin_box_add_static(hbox, _("Allows "), 0, SS_LEFT, FALSE, FALSE,
 			       5);
-	  fcwin_box_add_button(hbox, improvement_name_translation(j),
-			       is_great_wonder(j) ?
-			       ID_HELP_WONDER_LINK : ID_HELP_IMPROVEMENT_LINK,
-			       0 , FALSE, FALSE, 5);
+	  fcwin_box_add_button(hbox,
+			       improvement_name_translation(pimprove),
+			       is_great_wonder(pimprove)
+			       ? ID_HELP_WONDER_LINK
+			       : ID_HELP_IMPROVEMENT_LINK,
+			       0, FALSE, FALSE, 5);
 	}
-      }
-      if(i==improvement_by_number(j)->obsolete_by) {
+      } requirement_vector_iterate_end;
+      if (padvance == pimprove->obsolete_by) {
 	hbox=fcwin_hbox_new(helpdlg_win,FALSE);
 	fcwin_box_add_box(helpdlg_page_vbox,hbox,FALSE,FALSE,5);
 	fcwin_box_add_static(hbox,_("Obsoletes "),0,SS_LEFT,FALSE,FALSE,5);
-	fcwin_box_add_button(hbox, improvement_name_translation(j),
-			     is_great_wonder(j)?
-			     ID_HELP_WONDER_LINK:ID_HELP_IMPROVEMENT_LINK,
-			     0,FALSE,FALSE,5);
-      } requirement_vector_iterate_end;
-    } impr_type_iterate_end;
+	fcwin_box_add_button(hbox,
+			     improvement_name_translation(pimprove),
+			     is_great_wonder(pimprove)
+			     ? ID_HELP_WONDER_LINK
+			     : ID_HELP_IMPROVEMENT_LINK,
+			     0, FALSE, FALSE, 5);
+      }
+    } improvement_iterate_end;
 
-    unit_type_iterate(j) {
-      if (i != j->tech_requirement) continue;
+    unit_type_iterate(punittype) {
+      if (padvance != punittype->require_advance) {
+	continue;
+      }
       hbox=fcwin_hbox_new(helpdlg_win,FALSE);
       fcwin_box_add_box(helpdlg_page_vbox,hbox,FALSE,FALSE,5);
       fcwin_box_add_static(hbox,_("Allows "),0,SS_LEFT,FALSE,FALSE,5);
-      fcwin_box_add_button(hbox,utype_name_translation(j),
+      fcwin_box_add_button(hbox,
+			   utype_name_translation(punittype),
 			   ID_HELP_UNIT_LINK,
 			   0,FALSE,FALSE,5);
     } unit_type_iterate_end;
 
-    for (j = 0; j < game.control.num_tech_types; j++) {
-      if(i==advances[j].req[0]) {
-        if(advances[j].req[1]==A_NONE) {
+    advance_iterate(A_NONE, ptest) {
+      if (padvance == advance_requires(ptest, AR_ONE)) {
+	if (advance_by_number(A_NONE) == advance_requires(ptest, AR_TWO)) {
 	  hbox=fcwin_hbox_new(helpdlg_win,FALSE);
 	  fcwin_box_add_box(helpdlg_page_vbox,hbox,FALSE,FALSE,5);
 	  fcwin_box_add_static(hbox,_("Allows "),0,SS_LEFT,FALSE,FALSE,5);
-	  fcwin_box_add_button(hbox,advance_name_translation(j),
+	  fcwin_box_add_button(hbox,advance_name_translation(ptest),
 			       ID_HELP_TECH_LINK,0,FALSE,FALSE,5);
 	} else {
 	  hbox=fcwin_hbox_new(helpdlg_win,FALSE);
 	  fcwin_box_add_box(helpdlg_page_vbox,hbox,FALSE,FALSE,5);
 	  fcwin_box_add_static(hbox,_("Allows "),0,SS_LEFT,FALSE,FALSE,5);
-	  fcwin_box_add_button(hbox,advance_name_translation(j),
+	  fcwin_box_add_button(hbox,advance_name_translation(ptest),
 			       ID_HELP_TECH_LINK,0,FALSE,FALSE,5);
 	  fcwin_box_add_static(hbox,_(" (with "),0,SS_LEFT,FALSE,FALSE,5);
-	  fcwin_box_add_button(hbox,advance_name_translation(advances[j].req[1]),
+	  fcwin_box_add_button(hbox,advance_name_translation(advance_requires(ptest, AR_TWO)),
 			       ID_HELP_TECH_LINK,0,FALSE,FALSE,5);
 	  fcwin_box_add_static(hbox,Q_("?techhelp:)."),
 			       0,SS_LEFT,FALSE,FALSE,5);
 	}
       }
-      if (i==advances[j].req[1]) {
+      if (padvance == advance_requires(ptest, AR_TWO)) {
 	hbox=fcwin_hbox_new(helpdlg_win,FALSE);
 	fcwin_box_add_box(helpdlg_page_vbox,hbox,FALSE,FALSE,5);
 	fcwin_box_add_static(hbox,_("Allows "),0,SS_LEFT,FALSE,FALSE,5);
-	fcwin_box_add_button(hbox,advance_name_translation(j),
+	fcwin_box_add_button(hbox,advance_name_translation(ptest),
 			     ID_HELP_TECH_LINK,0,FALSE,FALSE,5);
 	fcwin_box_add_static(hbox,_(" (with "),0,SS_LEFT,FALSE,FALSE,5);
-	fcwin_box_add_button(hbox,advance_name_translation(advances[j].req[0]),
+	fcwin_box_add_button(hbox,advance_name_translation(advance_requires(ptest, AR_ONE)),
 			     ID_HELP_TECH_LINK,0,FALSE,FALSE,5);
 	fcwin_box_add_static(hbox,Q_("?techhelp:)."),
 			     0,SS_LEFT,FALSE,FALSE,5);
       }
-    }
+    } advance_iterate_end;
   }
 }
 
@@ -836,14 +873,15 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
   This is currently just a text page, with special text:
 **************************************************************************/
 static void help_update_government(const struct help_item *pitem,
-                                   char *title, struct government *gov)
+                                   char *title)
 {
   char buf[64000];
- 
+  struct government *gov = find_government_by_translated_name(title);
+
   if (!gov) {
     strcat(buf, pitem->text);
   } else {
-    helptext_government(buf, sizeof(buf), gov, pitem->text);
+    helptext_government(buf, sizeof(buf), client.conn.playing, pitem->text, gov);
   }
   create_help_page(HELP_TEXT);
   set_help_text(buf);
@@ -854,7 +892,6 @@ static void help_update_government(const struct help_item *pitem,
 **************************************************************************/
 static void help_update_dialog(const struct help_item *pitem)
 {
-  int i;
   char *top;
   /* figure out what kind of item is required for pitem ingo */
 
@@ -865,26 +902,22 @@ static void help_update_dialog(const struct help_item *pitem)
 
   switch(pitem->type) {
   case HELP_IMPROVEMENT:
-    i = find_improvement_by_translated_name(top);
-    if(i!=B_LAST && is_great_wonder(i)) i = B_LAST;
-    help_update_improvement(pitem, top, i);
+    help_update_improvement(pitem, top);
     break;
   case HELP_WONDER:
-    i = find_improvement_by_translated_name(top);
-    if(i!=B_LAST && !is_great_wonder(i)) i = B_LAST;
-    help_update_wonder(pitem, top, i);
+    help_update_wonder(pitem, top);
     break;
   case HELP_UNIT:
-    help_update_unit_type(pitem, top, find_unit_type_by_translated_name(top));
+    help_update_unit_type(pitem, top);
     break;
   case HELP_TECH:
-    help_update_tech(pitem, top, find_advance_by_translated_name(top));
+    help_update_tech(pitem, top);
     break;
   case HELP_TERRAIN:
-    help_update_terrain(pitem, top, find_terrain_by_translated_name(top));
+    help_update_terrain(pitem, top);
     break;
   case HELP_GOVERNMENT:
-    help_update_government(pitem, top, find_government_by_translated_name(top));
+    help_update_government(pitem, top);
     break;
   case HELP_TEXT:
   default:
