@@ -32,17 +32,23 @@
 #include "canvas.h"
 #include "pixcomm.h"
 
+/* utility */
 #include "fcintl.h"
-#include "game.h"
-#include "map.h"
+#include "log.h"
 #include "mem.h"
-#include "packets.h"
-#include "player.h"
 #include "shared.h"
-#include "spaceship.h"
 #include "support.h"
 
-#include "clinet.h"
+/* common */
+#include "game.h"
+#include "map.h"
+#include "packets.h"
+#include "player.h"
+#include "spaceship.h"
+
+/* client */
+#include "client_main.h"
+#include "climisc.h"
 #include "colors.h"
 #include "dialogs.h"
 #include "graphics.h"
@@ -55,7 +61,6 @@
 #include "repodlgs.h"
 #include "text.h"
 #include "tilespec.h"
-#include "climisc.h"
 
 #include "spaceshipdlg.h"
 
@@ -124,7 +129,7 @@ void refresh_spaceship_dialog(struct player *pplayer)
   pship=&(pdialog->pplayer->spaceship);
 
   if (game.info.spacerace
-     && pplayer->player_no == game.info.player_idx
+     && pplayer == client.conn.playing
      && pship->state == SSHIP_STARTED
      && pship->success_rate > 0) {
     XtSetSensitive(pdialog->launch_command, TRUE);
@@ -323,6 +328,6 @@ void spaceship_close_callback(Widget w, XtPointer client_data, XtPointer call_da
 void spaceship_launch_callback(Widget w, XtPointer client_data,
 			       XtPointer call_data)
 {
-  send_packet_spaceship_launch(&aconnection);
+  send_packet_spaceship_launch(&client.conn);
   /* close_spaceship_dialog((struct spaceship_dialog *)client_data); */
 }
