@@ -54,11 +54,6 @@ bool client_handle_packet(enum packet_type type, void *packet)
     handle_server_shutdown();
     return TRUE;
 
-  case PACKET_GAME_STATE:
-    handle_game_state(
-      ((struct packet_game_state *)packet)->value);
-    return TRUE;
-
   case PACKET_ENDGAME_REPORT:
     handle_endgame_report(packet);
     return TRUE;
@@ -93,6 +88,11 @@ bool client_handle_packet(enum packet_type type, void *packet)
       ((struct packet_chat_msg *)packet)->conn_id);
     return TRUE;
 
+  case PACKET_CONNECT_MSG:
+    handle_connect_msg(
+      ((struct packet_connect_msg *)packet)->message);
+    return TRUE;
+
   case PACKET_CITY_REMOVE:
     handle_city_remove(
       ((struct packet_city_remove *)packet)->city_id);
@@ -121,7 +121,7 @@ bool client_handle_packet(enum packet_type type, void *packet)
 
   case PACKET_PLAYER_REMOVE:
     handle_player_remove(
-      ((struct packet_player_remove *)packet)->player_id);
+      ((struct packet_player_remove *)packet)->playerno);
     return TRUE;
 
   case PACKET_PLAYER_INFO:
@@ -301,6 +301,14 @@ bool client_handle_packet(enum packet_type type, void *packet)
     handle_ruleset_terrain(packet);
     return TRUE;
 
+  case PACKET_RULESET_UNIT_CLASS:
+    handle_ruleset_unit_class(packet);
+    return TRUE;
+
+  case PACKET_RULESET_BASE:
+    handle_ruleset_base(packet);
+    return TRUE;
+
   case PACKET_RULESET_CONTROL:
     handle_ruleset_control(packet);
     return TRUE;
@@ -338,6 +346,10 @@ bool client_handle_packet(enum packet_type type, void *packet)
     handle_ruleset_resource(packet);
     return TRUE;
 
+  case PACKET_SCENARIO_INFO:
+    handle_scenario_info(packet);
+    return TRUE;
+
   case PACKET_VOTE_NEW:
     handle_vote_new(packet);
     return TRUE;
@@ -362,6 +374,12 @@ bool client_handle_packet(enum packet_type type, void *packet)
       ((struct packet_vote_resolve *)packet)->passed);
     return TRUE;
 
+  case PACKET_EDIT_OBJECT_CREATED:
+    handle_edit_object_created(
+      ((struct packet_edit_object_created *)packet)->tag,
+      ((struct packet_edit_object_created *)packet)->id);
+    return TRUE;
+
   default:
     return FALSE;
   }
