@@ -22,7 +22,7 @@ generate_stats=0
 
 # generate_freelogs will generate freelog calls to debug the delta code.
 generate_freelogs=0
-freelog_log_level="LOG_NORMAL"
+freelog_log_level="LOG_PACKET"
 generate_variant_freelogs=0
 
 ### The following parameters CHANGE the protocol. You have been warned.
@@ -223,7 +223,7 @@ class Field:
             else:     return "char *"
             
         if self.dataio_type=="worklist":
-            return "%s *"%self.struct_type
+            return "const %s *"%self.struct_type
         if self.is_array:
             return "%s *"%self.struct_type
         return self.struct_type+" "
@@ -242,7 +242,7 @@ class Field:
     # functions in the packet struct.
     def get_fill(self):
         if self.dataio_type=="worklist":
-            return "  copy_worklist(&real_packet->%(name)s, %(name)s);"%self.__dict__
+            return "  worklist_copy(&real_packet->%(name)s, %(name)s);"%self.__dict__
         if self.is_array==0:
             return "  real_packet->%(name)s = %(name)s;"%self.__dict__
         if self.dataio_type=="string":
@@ -419,10 +419,7 @@ class Field:
 
         if self.dataio_type=="bitvector":
             return "DIO_BV_GET(&din, real_packet->%(name)s);"%self.__dict__
-
-        if self.dataio_type=="memory":
-            return "dio_get_%(dataio_type)s(&din, real_packet->%(name)s, %(array_size_u)s);"%self.__dict__
-        if self.dataio_type in ["string","bit_string","city_map","memory"] and \
+        if self.dataio_type in ["string","bit_string","city_map"] and \
            self.is_array!=2:
             return "dio_get_%(dataio_type)s(&din, real_packet->%(name)s, sizeof(real_packet->%(name)s));"%self.__dict__
         if self.is_struct and self.is_array==0:
@@ -464,7 +461,7 @@ class Field:
             array_size_u=self.array_size_u
             array_size_d=self.array_size_d
 
-        if not self.diff:
+        if not self.diff or self.dataio_type=="memory":
             if array_size_u != array_size_d:
                 extra='''
   if(%(array_size_u)s > %(array_size_d)s) {
@@ -472,7 +469,10 @@ class Field:
     %(array_size_u)s = %(array_size_d)s;
   }'''%self.get_dict(vars())
             else:
-                extra=""                
+                extra=""
+            if self.dataio_type=="memory":
+                return '''%(extra)s
+  dio_get_%(dataio_type)s(&din, real_packet->%(name)s, %(array_size_u)s);'''%self.get_dict(vars())
             return '''
 {
   int i;
@@ -544,6 +544,7 @@ class Variant:
         if len(self.fields)>5 or string.split(self.name,"_")[1]=="ruleset":
             self.handle_via_packet=1
 
+        self.extra_force_arg=""
         self.extra_send_args=""
         self.extra_send_args2=""
         self.extra_send_args3=string.join(
@@ -555,9 +556,10 @@ class Variant:
         if not self.no_packet:
             self.extra_send_args=', const struct %(packet_name)s *packet'%self.__dict__+self.extra_send_args
             self.extra_send_args2=', packet'+self.extra_send_args2
-
+        if not self.is_action:
+            self.extra_force_arg=', bool force_send'
         self.receive_prototype='static struct %(packet_name)s *receive_%(name)s(struct connection *pc, enum packet_type type)'%self.__dict__
-        self.send_prototype='static int send_%(name)s(struct connection *pc%(extra_send_args)s)'%self.__dict__
+        self.send_prototype='static int send_%(name)s(struct connection *pc%(extra_force_arg)s%(extra_send_args)s)'%self.__dict__
 
     # See Field.get_dict
     def get_dict(self,vars):
@@ -590,11 +592,11 @@ static char *stats_%(name)s_names[] = {%(names)s};
         return '''
   if (stats_%(name)s_sent > 0 &&
       stats_%(name)s_discarded != stats_%(name)s_sent) {
-    freelog(LOG_NORMAL, \"%(name)s %%d out of %%d got discarded\",
+    freelog(LOG_TEST, \"%(name)s %%d out of %%d got discarded\",
       stats_%(name)s_discarded, stats_%(name)s_sent);
     for (i = 0; i < %(bits)d; i++) {
       if(stats_%(name)s_counters[i] > 0) {
-        freelog(LOG_NORMAL, \"  %%4d / %%4d: %%2d = %%s\",
+        freelog(LOG_TEST, \"  %%4d / %%4d: %%2d = %%s\",
           stats_%(name)s_counters[i],
           (stats_%(name)s_sent - stats_%(name)s_discarded),
           i, stats_%(name)s_names[i]);
@@ -714,12 +716,12 @@ static char *stats_%(name)s_names[] = {%(names)s};
             if self.delta:
                 body=self.get_delta_send_body()
                 if self.is_action:
-                    force_send="TRUE"
+                    force_send_var="\n  bool force_send = TRUE;"
                 else:
-                    force_send="FALSE"
+                    force_send_var=""
                 delta_header='''  %(name)s_fields fields;
   struct %(packet_name)s *old, *clone;
-  bool differ, old_from_hash, force_send_of_unchanged = %(force_send)s;
+  bool differ, old_from_hash;%(force_send_var)s
   struct hash_table **hash = &pc->phs.sent[%(type)s];
   int different = 0;
 '''
@@ -762,7 +764,7 @@ static char *stats_%(name)s_names[] = {%(names)s};
   if (!old) {
     old = fc_malloc(sizeof(*old));
     memset(old, 0, sizeof(*old));
-    force_send_of_unchanged = TRUE;
+    force_send = TRUE;
   }
 
 '''
@@ -778,7 +780,7 @@ static char *stats_%(name)s_names[] = {%(names)s};
             s='    stats_%(name)s_discarded++;\n'
         else:
             s=""
-        body=body+'''  if (different == 0 && !force_send_of_unchanged) {
+        body=body+'''  if (different == 0 && !force_send) {
 %(fl)s%(s)s<pre2>    return 0;
   }
 
@@ -926,7 +928,7 @@ class Packet:
             self.dirs.append("cs")
             arr.remove("cs")
         assert len(self.dirs)>0,repr(self.name)+repr(self.dirs)
-            
+
         self.is_action="is-info" not in arr
         if not self.is_action: arr.remove("is-info")
         
@@ -1001,11 +1003,16 @@ class Packet:
             self.extra_send_args2=', packet'+self.extra_send_args2
 
         self.receive_prototype='struct %(name)s *receive_%(name)s(struct connection *pc, enum packet_type type)'%self.__dict__
-        self.send_prototype='int send_%(name)s(struct connection *pc%(extra_send_args)s)'%self.__dict__
+        self.force_arg=""
+        self.force_value=""
+        if not self.is_action:
+            self.force_arg=", bool force_send"
+            self.force_value=", force_send"
+        self.send_prototype='int send_%(name)s(struct connection *pc%(force_arg)s%(extra_send_args)s)'%self.__dict__
         if self.want_lsend:
-            self.lsend_prototype='void lsend_%(name)s(struct conn_list *dest%(extra_send_args)s)'%self.__dict__
+            self.lsend_prototype='void lsend_%(name)s(struct conn_list *dest%(force_arg)s%(extra_send_args)s)'%self.__dict__
         if self.want_dsend:
-            self.dsend_prototype='int dsend_%(name)s(struct connection *pc%(extra_send_args3)s)'%self.__dict__
+            self.dsend_prototype='int dsend_%(name)s(struct connection *pc%(force_arg)s%(extra_send_args3)s)'%self.__dict__
             if self.want_lsend:
                 self.dlsend_prototype='void dlsend_%(name)s(struct conn_list *dest%(extra_send_args3)s)'%self.__dict__
 
@@ -1085,7 +1092,7 @@ class Packet:
             no=v.no
             result=result+'  } else if(%(cond)s) {\n    variant = %(no)s;\n'%self.get_dict(vars())
         if generate_variant_freelogs and len(self.variants)>1:
-            log='  freelog(LOG_NORMAL, "%(name)s: using variant=%%d cap=%%s", variant, pc->capability);\n'%self.get_dict(vars())
+            log='  freelog(LOG_TEST, "%(name)s: using variant=%%d cap=%%s", variant, pc->capability);\n'%self.get_dict(vars())
         else:
             log=""
         result=result+'''  } else {
@@ -1104,12 +1111,12 @@ class Packet:
         only_client=len(self.dirs)==1 and self.dirs[0]=="sc"
         only_server=len(self.dirs)==1 and self.dirs[0]=="cs"
         if only_client:
-            restrict='''  if (pc->is_server) {
+            restrict='''  if (is_server()) {
     freelog(LOG_ERROR, "Receiving %(name)s at the server.");
   }
 '''%self.get_dict(vars())
         elif only_server:
-            restrict='''  if (!pc->is_server) {
+            restrict='''  if (!is_server()) {
     freelog(LOG_ERROR, "Receiving %(name)s at the client.");
   }
 '''%self.get_dict(vars())
@@ -1140,12 +1147,12 @@ class Packet:
         only_client=len(self.dirs)==1 and self.dirs[0]=="cs"
         only_server=len(self.dirs)==1 and self.dirs[0]=="sc"
         if only_client:
-            restrict='''  if (pc->is_server) {
+            restrict='''  if (is_server()) {
     freelog(LOG_ERROR, "Sending %(name)s from the server.");
   }
 '''%self.get_dict(vars())
         elif only_server:
-            restrict='''  if (!pc->is_server) {
+            restrict='''  if (!is_server()) {
     freelog(LOG_ERROR, "Sending %(name)s from the client.");
   }
 '''%self.get_dict(vars())
@@ -1165,10 +1172,11 @@ class Packet:
 
   switch(pc->phs.variant[%(type)s]) {
 '''%self.get_dict(vars())
+        args="pc"
+        if not self.is_action:
+            args=args+', force_send'
         if not self.no_packet:
-            args="pc, packet"
-        else:
-            args="pc"
+            args=args+', packet'
         for v in self.variants:
             name2=v.name
             no=v.no
@@ -1195,7 +1203,7 @@ class Packet:
         return '''%(lsend_prototype)s
 {
   conn_list_iterate(dest, pconn) {
-    send_%(name)s(pconn%(extra_send_args2)s);
+    send_%(name)s(pconn%(force_value)s%(extra_send_args2)s);
   } conn_list_iterate_end;
 }
 
@@ -1212,7 +1220,7 @@ class Packet:
 
 %(fill)s
   
-  return send_%(name)s(pc, real_packet);
+  return send_%(name)s(pc, real_packet%(force_value)s);
 }
 
 '''%self.get_dict(vars())
@@ -1358,7 +1366,16 @@ def strip_c_comment(s):
 # various files.
 def main():
     ### parsing input
-    input_name="packets.def"
+    src_dir=os.path.dirname(sys.argv[0])
+    src_root=src_dir+"/.."
+    input_name=src_dir+"/packets.def"
+    ### We call this variable target_root instead of build_root
+    ### to avoid confusion as we are not building to builddir in
+    ### automake sense.
+    ### We build to src dir. Building to builddir causes a lot
+    ### of problems we have been unable to solve.
+    target_root=src_root
+
     content=open(input_name).read()
     content=strip_c_comment(content)
     lines=string.split(content,"\n")
@@ -1383,7 +1400,7 @@ def main():
     ### parsing finished
 
     ### writing packets_gen.h
-    output_h_name="packets_gen.h"
+    output_h_name=target_root+"/common/packets_gen.h"
 
     if lazy_overwrite:
         output_h=my_open(output_h_name+".tmp")
@@ -1407,7 +1424,7 @@ void *get_packet_from_connection_helper(struct connection *pc, enum packet_type
     output_h.close()
 
     ### writing packets_gen.c
-    output_c_name="packets_gen.c"
+    output_c_name=target_root+"/common/packets_gen.c"
     if lazy_overwrite:
         output_c=my_open(output_c_name+".tmp")
     else:
@@ -1421,15 +1438,19 @@ void *get_packet_from_connection_helper(struct connection *pc, enum packet_type
 #include <assert.h>
 #include <string.h>
 
+/* utility */
 #include "capability.h"
-#include "capstr.h"
-#include "connection.h"
-#include "dataio.h"
 #include "hash.h"
 #include "log.h"
 #include "mem.h"
 #include "support.h"
 
+/* common */
+#include "capstr.h"
+#include "connection.h"
+#include "dataio.h"
+#include "game.h"
+
 #include "packets.h"
 
 static unsigned int hash_const(const void *vkey, unsigned int num_buckets)
@@ -1483,7 +1504,7 @@ static int stats_total_sent;
                 open(i,"w").write(new)
             os.remove(i+".tmp")
 
-    f=my_open("../server/hand_gen.h")
+    f=my_open(target_root+"/server/hand_gen.h")
     f.write('''
 #ifndef FC__HAND_GEN_H
 #define FC__HAND_GEN_H
@@ -1525,7 +1546,7 @@ bool server_handle_packet(enum packet_type type, void *packet,
 ''')
     f.close()
 
-    f=my_open("../client/packhand_gen.h")
+    f=my_open(target_root+"/client/packhand_gen.h")
     f.write('''
 #ifndef FC__PACKHAND_GEN_H
 #define FC__PACKHAND_GEN_H
@@ -1556,7 +1577,7 @@ bool client_handle_packet(enum packet_type type, void *packet);
 ''')
     f.close()
 
-    f=my_open("../server/hand_gen.c")
+    f=my_open(target_root+"/server/hand_gen.c")
     f.write('''
 
 #ifdef HAVE_CONFIG_H
@@ -1611,7 +1632,7 @@ bool server_handle_packet(enum packet_type type, void *packet,
 ''')
     f.close()
 
-    f=my_open("../client/packhand_gen.c")
+    f=my_open(target_root+"/client/packhand_gen.c")
     f.write('''
 
 #ifdef HAVE_CONFIG_H
